@@ -33,6 +33,7 @@
 #include "commctrl.h"
 #include "advpub.h"
 #include "wininet.h"
+#include "pathcch.h"
 #include "shellapi.h"
 #include "urlmon.h"
 #include "msi.h"
@@ -57,10 +58,10 @@ WINE_DEFAULT_DEBUG_CHANNEL(appwizcpl);
 #define GECKO_SHA "???"
 #endif
 
-#define MONO_VERSION "6.0.0"
+#define MONO_VERSION "6.4.0"
 #if defined(__i386__) || defined(__x86_64__)
 #define MONO_ARCH "x86"
-#define MONO_SHA "5c7af5976d101be359a77045c9cc8fc40feac028263d00e731578864e8b183d0"
+#define MONO_SHA "db8b85f3b910590f61adbfa0005a995d613e299a1cc73d00553d70dc81010e36"
 #else
 #define MONO_ARCH ""
 #define MONO_SHA "???"
@@ -197,10 +198,11 @@ static enum install_res install_file(const WCHAR *file_name)
 
 static enum install_res install_from_dos_file(const WCHAR *dir, const WCHAR *subdir, const WCHAR *file_name)
 {
-    WCHAR *path;
+    WCHAR *path, *canonical_path;
     enum install_res ret;
     int len = lstrlenW( dir );
     int size = len + 1;
+    HRESULT hr;
 
     size += lstrlenW( subdir ) + lstrlenW( file_name ) + 2;
     if (!(path = heap_alloc( size * sizeof(WCHAR) ))) return INSTALL_FAILED;
@@ -213,16 +215,25 @@ static enum install_res install_from_dos_file(const WCHAR *dir, const WCHAR *sub
     lstrcatW( path, L"\\" );
     lstrcatW( path, file_name );
 
-    if (GetFileAttributesW( path ) == INVALID_FILE_ATTRIBUTES)
+    hr = PathAllocCanonicalize( path, PATHCCH_ALLOW_LONG_PATHS, &canonical_path );
+    if (FAILED( hr ))
     {
-        TRACE( "%s not found\n", debugstr_w(path) );
+        ERR( "Failed to canonicalize %s, hr %#x\n", debugstr_w(path), hr );
         heap_free( path );
         return INSTALL_NEXT;
     }
-
-    ret = install_file( path );
-
     heap_free( path );
+
+    if (GetFileAttributesW( canonical_path ) == INVALID_FILE_ATTRIBUTES)
+    {
+        TRACE( "%s not found\n", debugstr_w(canonical_path) );
+        LocalFree( canonical_path );
+        return INSTALL_NEXT;
+    }
+
+    ret = install_file( canonical_path );
+
+    LocalFree( canonical_path );
     return ret;
 }
 
