@@ -101,7 +101,6 @@ WINE_DEFAULT_DEBUG_CHANNEL(dinput);
 static CFMutableArrayRef device_main_elements = NULL;
 
 typedef struct JoystickImpl JoystickImpl;
-static const IDirectInputDevice8AVtbl JoystickAvt;
 static const IDirectInputDevice8WVtbl JoystickWvt;
 
 struct JoystickImpl
@@ -116,22 +115,12 @@ struct JoystickImpl
     struct list effects;
 };
 
-static inline JoystickImpl *impl_from_IDirectInputDevice8A(IDirectInputDevice8A *iface)
-{
-    return CONTAINING_RECORD(CONTAINING_RECORD(CONTAINING_RECORD(iface, IDirectInputDeviceImpl, IDirectInputDevice8A_iface),
-           JoystickGenericImpl, base), JoystickImpl, generic);
-}
 static inline JoystickImpl *impl_from_IDirectInputDevice8W(IDirectInputDevice8W *iface)
 {
     return CONTAINING_RECORD(CONTAINING_RECORD(CONTAINING_RECORD(iface, IDirectInputDeviceImpl, IDirectInputDevice8W_iface),
            JoystickGenericImpl, base), JoystickImpl, generic);
 }
 
-static inline IDirectInputDevice8W *IDirectInputDevice8W_from_impl(JoystickImpl *This)
-{
-    return &This->generic.base.IDirectInputDevice8W_iface;
-}
-
 typedef struct _EffectImpl {
     IDirectInputEffect IDirectInputEffect_iface;
     LONG ref;
@@ -797,9 +786,9 @@ static void get_osx_device_elements_props(JoystickImpl *device)
     }
 }
 
-static void poll_osx_device_state(LPDIRECTINPUTDEVICE8A iface)
+static void poll_osx_device_state( IDirectInputDevice8W *iface )
 {
-    JoystickImpl *device = impl_from_IDirectInputDevice8A(iface);
+    JoystickImpl *device = impl_from_IDirectInputDevice8W( iface );
     IOHIDElementRef device_main_element;
     IOHIDDeviceRef hid_device;
 
@@ -851,6 +840,8 @@ static void poll_osx_device_state(LPDIRECTINPUTDEVICE8A iface)
                         {
                             inst_id = DIDFT_MAKEINSTANCE(button_idx) | DIDFT_PSHBUTTON;
                             queue_event(iface,inst_id,newVal,GetCurrentTime(),device->generic.base.dinput->evsequence++);
+                            if (device->generic.base.hEvent)
+                                SetEvent( device->generic.base.hEvent );
                         }
                         button_idx ++;
                     }
@@ -881,6 +872,8 @@ static void poll_osx_device_state(LPDIRECTINPUTDEVICE8A iface)
                             {
                                 inst_id = DIDFT_MAKEINSTANCE(pov_idx) | DIDFT_POV;
                                 queue_event(iface,inst_id,newVal,GetCurrentTime(),device->generic.base.dinput->evsequence++);
+                                if (device->generic.base.hEvent)
+                                    SetEvent( device->generic.base.hEvent );
                             }
                             pov_idx ++;
                             break;
@@ -958,6 +951,8 @@ static void poll_osx_device_state(LPDIRECTINPUTDEVICE8A iface)
                             {
                                 inst_id = DIDFT_MAKEINSTANCE(wine_obj) | DIDFT_ABSAXIS;
                                 queue_event(iface,inst_id,newVal,GetCurrentTime(),device->generic.base.dinput->evsequence++);
+                                if (device->generic.base.hEvent)
+                                    SetEvent( device->generic.base.hEvent );
                             }
 
                             break;
@@ -995,53 +990,7 @@ static DWORD make_vid_pid(IOHIDDeviceRef device)
     return MAKELONG(vendID, prodID);
 }
 
-static HRESULT joydev_enum_deviceA(DWORD dwDevType, DWORD dwFlags, LPDIDEVICEINSTANCEA lpddi, DWORD version, int id)
-{
-    IOHIDDeviceRef device;
-    BOOL is_joystick;
-
-    TRACE("dwDevType %u dwFlags 0x%08x version 0x%04x id %d\n", dwDevType, dwFlags, version, id);
-
-    if (id >= find_joystick_devices()) return E_FAIL;
-
-    device = get_device_ref(id);
-
-    if ((dwDevType == 0) ||
-    ((dwDevType == DIDEVTYPE_JOYSTICK) && (version >= 0x0300 && version < 0x0800)) ||
-    (((dwDevType == DI8DEVCLASS_GAMECTRL) || (dwDevType == DI8DEVTYPE_JOYSTICK)) && (version >= 0x0800)))
-    {
-        if (dwFlags & DIEDFL_FORCEFEEDBACK) {
-            if(!device)
-                return S_FALSE;
-            if(get_ff(device, NULL) != S_OK)
-                return S_FALSE;
-        }
-        is_joystick = get_device_property_long(device, CFSTR(kIOHIDDeviceUsageKey)) == kHIDUsage_GD_Joystick;
-        /* Return joystick */
-        lpddi->guidInstance = DInput_Wine_OsX_Joystick_GUID;
-        lpddi->guidInstance.Data3 = id;
-        lpddi->guidProduct = DInput_PIDVID_Product_GUID;
-        lpddi->guidProduct.Data1 = make_vid_pid(device);
-        lpddi->dwDevType = get_device_type(version, is_joystick);
-        lpddi->dwDevType |= DIDEVTYPE_HID;
-        lpddi->wUsagePage = 0x01; /* Desktop */
-        if (is_joystick)
-            lpddi->wUsage = 0x04; /* Joystick */
-        else
-            lpddi->wUsage = 0x05; /* Game Pad */
-        sprintf(lpddi->tszInstanceName, "Joystick %d", id);
-
-        /* get the device name */
-        get_osx_device_name(id, lpddi->tszProductName, MAX_PATH);
-
-        lpddi->guidFFDriver = GUID_NULL;
-        return S_OK;
-    }
-
-    return S_FALSE;
-}
-
-static HRESULT joydev_enum_deviceW(DWORD dwDevType, DWORD dwFlags, LPDIDEVICEINSTANCEW lpddi, DWORD version, int id)
+static HRESULT joydev_enum_device(DWORD dwDevType, DWORD dwFlags, LPDIDEVICEINSTANCEW lpddi, DWORD version, int id)
 {
     char name[MAX_PATH];
     char friendly[32];
@@ -1115,8 +1064,7 @@ static BOOL osx_axis_has_ff(FFCAPABILITIES *ffcaps, UInt8 axis)
     return FALSE;
 }
 
-static HRESULT alloc_device(REFGUID rguid, IDirectInputImpl *dinput,
-                            JoystickImpl **pdev, unsigned short index)
+static HRESULT alloc_device( REFGUID rguid, IDirectInputImpl *dinput, JoystickImpl **out, unsigned short index )
 {
     DWORD i;
     IOHIDDeviceRef device;
@@ -1129,14 +1077,12 @@ static HRESULT alloc_device(REFGUID rguid, IDirectInputImpl *dinput,
     int slider_count = 0;
     FFCAPABILITIES ffcaps;
 
-    TRACE("%s %p %p %hu\n", debugstr_guid(rguid), dinput, pdev, index);
+    TRACE( "%s %p %p %hu\n", debugstr_guid( rguid ), dinput, out, index );
 
-    newDevice = HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(JoystickImpl));
-    if (newDevice == 0) {
-        WARN("out of memory\n");
-        *pdev = 0;
-        return DIERR_OUTOFMEMORY;
-    }
+    if (FAILED(hr = direct_input_device_alloc( sizeof(JoystickImpl), &JoystickWvt, rguid, dinput, (void **)&newDevice )))
+        return hr;
+    df = newDevice->generic.base.data_format.wine_df;
+    newDevice->generic.base.crit.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": JoystickImpl*->generic.base.crit");
 
     newDevice->id = index;
 
@@ -1198,16 +1144,7 @@ static HRESULT alloc_device(REFGUID rguid, IDirectInputImpl *dinput,
         newDevice->generic.devcaps.dwButtons = 128;
     }
 
-    newDevice->generic.base.IDirectInputDevice8A_iface.lpVtbl = &JoystickAvt;
-    newDevice->generic.base.IDirectInputDevice8W_iface.lpVtbl = &JoystickWvt;
-    newDevice->generic.base.ref = 1;
-    newDevice->generic.base.dinput = dinput;
-    newDevice->generic.base.guid = *rguid;
-    InitializeCriticalSection(&newDevice->generic.base.crit);
-    newDevice->generic.base.crit.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": JoystickImpl*->generic.base.crit");
-
     /* Create copy of default data format */
-    if (!(df = HeapAlloc(GetProcessHeap(), 0, c_dfDIJoystick2.dwSize))) goto FAILED;
     memcpy(df, &c_dfDIJoystick2, c_dfDIJoystick2.dwSize);
 
     df->dwNumObjs = newDevice->generic.devcaps.dwAxes + newDevice->generic.devcaps.dwPOVs + newDevice->generic.devcaps.dwButtons;
@@ -1272,13 +1209,10 @@ static HRESULT alloc_device(REFGUID rguid, IDirectInputImpl *dinput,
         df->rgodf[idx  ].pguid = &GUID_Button;
         df->rgodf[idx++].dwType = DIDFT_MAKEINSTANCE(i) | DIDFT_PSHBUTTON;
     }
-    newDevice->generic.base.data_format.wine_df = df;
 
     /* initialize default properties */
     get_osx_device_elements_props(newDevice);
 
-    IDirectInput_AddRef(&newDevice->generic.base.dinput->IDirectInput7A_iface);
-
     newDevice->generic.devcaps.dwSize = sizeof(newDevice->generic.devcaps);
     newDevice->generic.devcaps.dwFlags |= DIDC_ATTACHED;
     if (newDevice->generic.base.dinput->dwVersion >= 0x0800)
@@ -1297,8 +1231,7 @@ static HRESULT alloc_device(REFGUID rguid, IDirectInputImpl *dinput,
         _dump_DIDEVCAPS(&newDevice->generic.devcaps);
     }
 
-    *pdev = newDevice;
-
+    *out = newDevice;
     return DI_OK;
 
 FAILED:
@@ -1310,8 +1243,6 @@ FAILED:
     release_DataFormat(&newDevice->generic.base.data_format);
     HeapFree(GetProcessHeap(),0,newDevice->generic.name);
     HeapFree(GetProcessHeap(),0,newDevice);
-    *pdev = 0;
-
     return hr;
 }
 
@@ -1351,13 +1282,13 @@ static unsigned short get_joystick_index(REFGUID guid)
     return 0xffff;
 }
 
-static HRESULT joydev_create_device(IDirectInputImpl *dinput, REFGUID rguid, REFIID riid, LPVOID *pdev, int unicode)
+static HRESULT joydev_create_device( IDirectInputImpl *dinput, REFGUID rguid, IDirectInputDevice8W **out )
 {
     unsigned short index;
     int joystick_devices_count;
 
-    TRACE("%p %s %s %p %i\n", dinput, debugstr_guid(rguid), debugstr_guid(riid), pdev, unicode);
-    *pdev = NULL;
+    TRACE( "%p %s %p\n", dinput, debugstr_guid( rguid ), out );
+    *out = NULL;
 
     if ((joystick_devices_count = find_joystick_devices()) == 0)
         return DIERR_DEVICENOTREG;
@@ -1368,35 +1299,11 @@ static HRESULT joydev_create_device(IDirectInputImpl *dinput, REFGUID rguid, REF
         JoystickImpl *This;
         HRESULT hr;
 
-        if (riid == NULL)
-            ;/* nothing */
-        else if (IsEqualGUID(&IID_IDirectInputDeviceA,  riid) ||
-                 IsEqualGUID(&IID_IDirectInputDevice2A, riid) ||
-                 IsEqualGUID(&IID_IDirectInputDevice7A, riid) ||
-                 IsEqualGUID(&IID_IDirectInputDevice8A, riid))
-        {
-            unicode = 0;
-        }
-        else if (IsEqualGUID(&IID_IDirectInputDeviceW,  riid) ||
-                 IsEqualGUID(&IID_IDirectInputDevice2W, riid) ||
-                 IsEqualGUID(&IID_IDirectInputDevice7W, riid) ||
-                 IsEqualGUID(&IID_IDirectInputDevice8W, riid))
-        {
-            unicode = 1;
-        }
-        else
-        {
-            WARN("no interface\n");
-            return DIERR_NOINTERFACE;
-        }
+        if (FAILED(hr = alloc_device( rguid, dinput, &This, index ))) return hr;
 
-        hr = alloc_device(rguid, dinput, &This, index);
-        if (!This) return hr;
+        TRACE( "Created a Joystick device (%p)\n", This );
 
-        if (unicode)
-            *pdev = &This->generic.base.IDirectInputDevice8W_iface;
-        else
-            *pdev = &This->generic.base.IDirectInputDevice8A_iface;
+        *out = &This->generic.base.IDirectInputDevice8W_iface;
         return hr;
     }
 
@@ -1444,12 +1351,6 @@ static HRESULT WINAPI JoystickWImpl_GetProperty(LPDIRECTINPUTDEVICE8W iface, REF
     return DI_OK;
 }
 
-static HRESULT WINAPI JoystickAImpl_GetProperty(LPDIRECTINPUTDEVICE8A iface, REFGUID rguid, LPDIPROPHEADER pdiph)
-{
-    JoystickImpl *This = impl_from_IDirectInputDevice8A(iface);
-    return JoystickWImpl_GetProperty(IDirectInputDevice8W_from_impl(This), rguid, pdiph);
-}
-
 static HRESULT osx_set_autocenter(JoystickImpl *This,
         const DIPROPDWORD *header)
 {
@@ -1493,24 +1394,6 @@ static HRESULT WINAPI JoystickWImpl_SetProperty(IDirectInputDevice8W *iface,
     return JoystickWGenericImpl_SetProperty(iface, prop, header);
 }
 
-static HRESULT WINAPI JoystickAImpl_SetProperty(IDirectInputDevice8A *iface,
-        const GUID *prop, const DIPROPHEADER *header)
-{
-    JoystickImpl *This = impl_from_IDirectInputDevice8A(iface);
-
-    TRACE("%p %s %p\n", This, debugstr_guid(prop), header);
-
-    switch(LOWORD(prop))
-    {
-    case (DWORD_PTR)DIPROP_AUTOCENTER:
-        return osx_set_autocenter(This, (const DIPROPDWORD *)header);
-    case (DWORD_PTR)DIPROP_FFGAIN:
-        return osx_set_ffgain(This, (const DIPROPDWORD *)header);
-    }
-
-    return JoystickAGenericImpl_SetProperty(iface, prop, header);
-}
-
 static CFUUIDRef effect_win_to_mac(const GUID *effect)
 {
 #define DO_MAP(X) \
@@ -1576,18 +1459,6 @@ static HRESULT WINAPI JoystickWImpl_CreateEffect(IDirectInputDevice8W *iface,
     return S_OK;
 }
 
-static HRESULT WINAPI JoystickAImpl_CreateEffect(IDirectInputDevice8A *iface,
-        const GUID *type, const DIEFFECT *params, IDirectInputEffect **out,
-        IUnknown *outer)
-{
-    JoystickImpl *This = impl_from_IDirectInputDevice8A(iface);
-
-    TRACE("(%p)->(%s %p %p %p)\n", This, debugstr_guid(type), params, out, outer);
-
-    return JoystickWImpl_CreateEffect(&This->generic.base.IDirectInputDevice8W_iface,
-            type, params, out, outer);
-}
-
 static HRESULT WINAPI JoystickWImpl_SendForceFeedbackCommand(IDirectInputDevice8W *iface,
         DWORD flags)
 {
@@ -1608,64 +1479,17 @@ static HRESULT WINAPI JoystickWImpl_SendForceFeedbackCommand(IDirectInputDevice8
     return S_OK;
 }
 
-static HRESULT WINAPI JoystickAImpl_SendForceFeedbackCommand(IDirectInputDevice8A *iface,
-        DWORD flags)
-{
-    JoystickImpl *This = impl_from_IDirectInputDevice8A(iface);
-
-    TRACE("%p 0x%x\n", This, flags);
-
-    return JoystickWImpl_SendForceFeedbackCommand(&This->generic.base.IDirectInputDevice8W_iface, flags);
-}
-
 const struct dinput_device joystick_osx_device = {
   "Wine OS X joystick driver",
-  joydev_enum_deviceA,
-  joydev_enum_deviceW,
+  joydev_enum_device,
   joydev_create_device
 };
 
-static const IDirectInputDevice8AVtbl JoystickAvt =
-{
-    IDirectInputDevice2AImpl_QueryInterface,
-    IDirectInputDevice2AImpl_AddRef,
-    IDirectInputDevice2AImpl_Release,
-    JoystickAGenericImpl_GetCapabilities,
-    IDirectInputDevice2AImpl_EnumObjects,
-    JoystickAImpl_GetProperty,
-    JoystickAImpl_SetProperty,
-    IDirectInputDevice2AImpl_Acquire,
-    IDirectInputDevice2AImpl_Unacquire,
-    JoystickAGenericImpl_GetDeviceState,
-    IDirectInputDevice2AImpl_GetDeviceData,
-    IDirectInputDevice2AImpl_SetDataFormat,
-    IDirectInputDevice2AImpl_SetEventNotification,
-    IDirectInputDevice2AImpl_SetCooperativeLevel,
-    JoystickAGenericImpl_GetObjectInfo,
-    JoystickAGenericImpl_GetDeviceInfo,
-    IDirectInputDevice2AImpl_RunControlPanel,
-    IDirectInputDevice2AImpl_Initialize,
-    JoystickAImpl_CreateEffect,
-    IDirectInputDevice2AImpl_EnumEffects,
-    IDirectInputDevice2AImpl_GetEffectInfo,
-    IDirectInputDevice2AImpl_GetForceFeedbackState,
-    JoystickAImpl_SendForceFeedbackCommand,
-    IDirectInputDevice2AImpl_EnumCreatedEffectObjects,
-    IDirectInputDevice2AImpl_Escape,
-    JoystickAGenericImpl_Poll,
-    IDirectInputDevice2AImpl_SendDeviceData,
-    IDirectInputDevice7AImpl_EnumEffectsInFile,
-    IDirectInputDevice7AImpl_WriteEffectToFile,
-    JoystickAGenericImpl_BuildActionMap,
-    JoystickAGenericImpl_SetActionMap,
-    IDirectInputDevice8AImpl_GetImageInfo
-};
-
 static const IDirectInputDevice8WVtbl JoystickWvt =
 {
     IDirectInputDevice2WImpl_QueryInterface,
     IDirectInputDevice2WImpl_AddRef,
-    IDirectInputDevice2WImpl_Release,
+    JoystickWGenericImpl_Release,
     JoystickWGenericImpl_GetCapabilities,
     IDirectInputDevice2WImpl_EnumObjects,
     JoystickWImpl_GetProperty,
@@ -1834,7 +1658,6 @@ const struct dinput_device joystick_osx_device = {
   "Wine OS X joystick driver",
   NULL,
   NULL,
-  NULL
 };
 
 #endif /* HAVE_IOHIDMANAGERCREATE */
