@@ -72,20 +72,17 @@
 #include "winternl.h"
 #include "ddk/d3dkmthk.h"
 
-#include "gdi_private.h"
+#include "ntgdi_private.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(bitmap);
 
 
-static HGDIOBJ DIB_SelectObject( HGDIOBJ handle, HDC hdc );
 static INT DIB_GetObject( HGDIOBJ handle, INT count, LPVOID buffer );
 static BOOL DIB_DeleteObject( HGDIOBJ handle );
 
 static const struct gdi_obj_funcs dib_funcs =
 {
-    DIB_SelectObject,  /* pSelectObject */
-    DIB_GetObject,     /* pGetObjectA */
     DIB_GetObject,     /* pGetObjectW */
     NULL,              /* pUnrealizeObject */
     DIB_DeleteObject   /* pDeleteObject */
@@ -96,7 +93,7 @@ static const struct gdi_obj_funcs dib_funcs =
  *
  * Return the size of the bitmap info structure including color table.
  */
-int bitmap_info_size( const BITMAPINFO * info, WORD coloruse )
+static int bitmap_info_size( const BITMAPINFO *info, WORD coloruse )
 {
     unsigned int colors, size, masks = 0;
 
@@ -256,7 +253,7 @@ static BOOL bitmapinfo_from_user_bitmapinfo( BITMAPINFO *dst, const BITMAPINFO *
 static int fill_color_table_from_palette( BITMAPINFO *info, HDC hdc )
 {
     PALETTEENTRY palEntry[256];
-    HPALETTE palette = GetCurrentObject( hdc, OBJ_PAL );
+    HPALETTE palette = NtGdiGetDCObject( hdc, NTGDI_OBJ_PAL );
     int i, colors = 1 << info->bmiHeader.biBitCount;
 
     info->bmiHeader.biClrUsed = colors;
@@ -264,7 +261,7 @@ static int fill_color_table_from_palette( BITMAPINFO *info, HDC hdc )
     if (!palette) return 0;
 
     memset( palEntry, 0, sizeof(palEntry) );
-    if (!GetPaletteEntries( palette, 0, colors, palEntry ))
+    if (!get_palette_entries( palette, 0, colors, palEntry ))
         return 0;
 
     for (i = 0; i < colors; i++)
@@ -287,8 +284,9 @@ BOOL fill_color_table_from_pal_colors( BITMAPINFO *info, HDC hdc )
     int i, count, colors = info->bmiHeader.biClrUsed;
 
     if (!colors) return TRUE;
-    if (!(palette = GetCurrentObject( hdc, OBJ_PAL ))) return FALSE;
-    if (!(count = GetPaletteEntries( palette, 0, colors, entries ))) return FALSE;
+    if (!(palette = NtGdiGetDCObject( hdc, NTGDI_OBJ_PAL )))
+        return FALSE;
+    if (!(count = get_palette_entries( palette, 0, colors, entries ))) return FALSE;
 
     for (i = 0; i < colors; i++, index++)
     {
@@ -332,8 +330,8 @@ static BOOL build_rle_bitmap( BITMAPINFO *info, struct gdi_image_bits *bits, HRG
 
     if (clip)
     {
-        *clip = CreateRectRgn( 0, 0, 0, 0 );
-        run   = CreateRectRgn( 0, 0, 0, 0 );
+        *clip = NtGdiCreateRectRgn( 0, 0, 0, 0 );
+        run   = NtGdiCreateRectRgn( 0, 0, 0, 0 );
         if (!*clip || !run) goto fail;
     }
 
@@ -377,8 +375,8 @@ static BOOL build_rle_bitmap( BITMAPINFO *info, struct gdi_image_bits *bits, HRG
             {
                 if(left != right && clip)
                 {
-                    SetRectRgn( run, left, y, right, y + 1 );
-                    CombineRgn( *clip, run, *clip, RGN_OR );
+                    NtGdiSetRectRgn( run, left, y, right, y + 1 );
+                    NtGdiCombineRgn( *clip, run, *clip, RGN_OR );
                 }
                 switch (data)
                 {
@@ -441,7 +439,7 @@ static BOOL build_rle_bitmap( BITMAPINFO *info, struct gdi_image_bits *bits, HRG
     }
 
 done:
-    if (run) DeleteObject( run );
+    if (run) NtGdiDeleteObjectApp( run );
     if (bits->free) bits->free( bits );
 
     bits->ptr     = out_bits;
@@ -452,8 +450,8 @@ done:
     return TRUE;
 
 fail:
-    if (run) DeleteObject( run );
-    if (clip && *clip) DeleteObject( *clip );
+    if (run) NtGdiDeleteObjectApp( run );
+    if (clip && *clip) NtGdiDeleteObjectApp( *clip );
     HeapFree( GetProcessHeap(), 0, out_bits );
     return FALSE;
 }
@@ -495,7 +493,7 @@ INT CDECL nulldrv_StretchDIBits( PHYSDEV dev, INT xDst, INT yDst, INT widthDst,
     dst.width  = rect.right - rect.left;
     dst.height = rect.bottom - rect.top;
 
-    if (dc->layout & LAYOUT_RTL && rop & NOMIRRORBITMAP)
+    if (dc->attr->layout & LAYOUT_RTL && rop & NOMIRRORBITMAP)
     {
         dst.x += dst.width;
         dst.width = -dst.width;
@@ -565,7 +563,7 @@ INT CDECL nulldrv_StretchDIBits( PHYSDEV dev, INT xDst, INT yDst, INT widthDst,
 
     if (!intersect_vis_rectangles( &dst, &src )) goto done;
 
-    if (clip) OffsetRgn( clip, dst.x - src.x, dst.y - src.y );
+    if (clip) NtGdiOffsetRgn( clip, dst.x - src.x, dst.y - src.y );
 
     dev = GET_DC_PHYSDEV( dc, pPutImage );
     copy_bitmapinfo( dst_info, src_info );
@@ -595,24 +593,24 @@ INT CDECL nulldrv_StretchDIBits( PHYSDEV dev, INT xDst, INT yDst, INT widthDst,
     if (err == ERROR_TRANSFORM_NOT_SUPPORTED)
     {
         copy_bitmapinfo( src_info, dst_info );
-        err = stretch_bits( src_info, &src, dst_info, &dst, &src_bits, dc->stretchBltMode );
+        err = stretch_bits( src_info, &src, dst_info, &dst, &src_bits, dc->attr->stretch_blt_mode );
         if (!err) err = dev->funcs->pPutImage( dev, NULL, dst_info, &src_bits, &src, &dst, rop );
     }
     if (err) ret = 0;
 
 done:
     if (src_bits.free) src_bits.free( &src_bits );
-    if (clip) DeleteObject( clip );
+    if (clip) NtGdiDeleteObjectApp( clip );
     return ret;
 }
 
 /***********************************************************************
- *           StretchDIBits   (GDI32.@)
+ *           NtGdiStretchDIBitsInternal   (win32u.@)
  */
-INT WINAPI DECLSPEC_HOTPATCH StretchDIBits( HDC hdc, INT xDst, INT yDst, INT widthDst, INT heightDst,
-                                            INT xSrc, INT ySrc, INT widthSrc, INT heightSrc,
-                                            const void *bits, const BITMAPINFO *bmi, UINT coloruse,
-                                            DWORD rop )
+INT WINAPI NtGdiStretchDIBitsInternal( HDC hdc, INT xDst, INT yDst, INT widthDst, INT heightDst,
+                                       INT xSrc, INT ySrc, INT widthSrc, INT heightSrc,
+                                       const void *bits, const BITMAPINFO *bmi, UINT coloruse,
+                                       DWORD rop, UINT max_info, UINT max_bits, HANDLE xform )
 {
     char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
     BITMAPINFO *info = (BITMAPINFO *)buffer;
@@ -695,7 +693,7 @@ INT WINAPI SetDIBits( HDC hdc, HBITMAP hbitmap, UINT startscan,
 
     if (coloruse == DIB_PAL_COLORS && !fill_color_table_from_pal_colors( src_info, hdc )) return 0;
 
-    if (!(bitmap = GDI_GetObjPtr( hbitmap, OBJ_BITMAP ))) return 0;
+    if (!(bitmap = GDI_GetObjPtr( hbitmap, NTGDI_OBJ_BITMAP ))) return 0;
 
     if (src_info->bmiHeader.biCompression == BI_RLE4 || src_info->bmiHeader.biCompression == BI_RLE8)
     {
@@ -758,7 +756,7 @@ INT WINAPI SetDIBits( HDC hdc, HBITMAP hbitmap, UINT startscan,
 
 done:
     if (src_bits.free) src_bits.free( &src_bits );
-    if (clip) DeleteObject( clip );
+    if (clip) NtGdiDeleteObjectApp( clip );
     GDI_ReleaseObj( hbitmap );
     return result;
 }
@@ -849,7 +847,7 @@ INT CDECL nulldrv_SetDIBitsToDevice( PHYSDEV dev, INT x_dst, INT y_dst, DWORD cx
     dst.y = pt.y;
     dst.width = cx;
     dst.height = cy;
-    if (dc->layout & LAYOUT_RTL) dst.x -= cx - 1;
+    if (dc->attr->layout & LAYOUT_RTL) dst.x -= cx - 1;
 
     rect.left = dst.x;
     rect.top = dst.y;
@@ -862,7 +860,7 @@ INT CDECL nulldrv_SetDIBitsToDevice( PHYSDEV dev, INT x_dst, INT y_dst, DWORD cx
     src.visrect = dst.visrect = rect;
     offset_rect( &src.visrect, src.x - dst.x, src.y - dst.y );
     if (is_rect_empty( &dst.visrect )) goto done;
-    if (clip) OffsetRgn( clip, dst.x - src.x, dst.y - src.y );
+    if (clip) NtGdiOffsetRgn( clip, dst.x - src.x, dst.y - src.y );
 
     dev = GET_DC_PHYSDEV( dc, pPutImage );
     copy_bitmapinfo( dst_info, src_info );
@@ -876,17 +874,18 @@ INT CDECL nulldrv_SetDIBitsToDevice( PHYSDEV dev, INT x_dst, INT y_dst, DWORD cx
 
 done:
     if (src_bits.free) src_bits.free( &src_bits );
-    if (clip) DeleteObject( clip );
+    if (clip) NtGdiDeleteObjectApp( clip );
     return lines;
 }
 
 /***********************************************************************
- *           SetDIBitsToDevice   (GDI32.@)
+ *           NtGdiSetDIBitsToDeviceInternal   (win32u.@)
  */
-INT WINAPI SetDIBitsToDevice(HDC hdc, INT xDest, INT yDest, DWORD cx,
-                           DWORD cy, INT xSrc, INT ySrc, UINT startscan,
-                           UINT lines, LPCVOID bits, const BITMAPINFO *bmi,
-                           UINT coloruse )
+INT WINAPI NtGdiSetDIBitsToDeviceInternal( HDC hdc, INT xDest, INT yDest, DWORD cx,
+                                           DWORD cy, INT xSrc, INT ySrc, UINT startscan,
+                                           UINT lines, const void *bits, const BITMAPINFO *bmi,
+                                           UINT coloruse, UINT max_bits, UINT max_info,
+                                           BOOL xform_coords, HANDLE xform )
 {
     char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
     BITMAPINFO *info = (BITMAPINFO *)buffer;
@@ -912,10 +911,7 @@ INT WINAPI SetDIBitsToDevice(HDC hdc, INT xDest, INT yDest, DWORD cx,
     return ret;
 }
 
-/***********************************************************************
- *           SetDIBColorTable    (GDI32.@)
- */
-UINT WINAPI SetDIBColorTable( HDC hdc, UINT startpos, UINT entries, const RGBQUAD *colors )
+UINT set_dib_dc_color_table( HDC hdc, UINT startpos, UINT entries, const RGBQUAD *colors )
 {
     DC * dc;
     UINT i, result = 0;
@@ -923,7 +919,7 @@ UINT WINAPI SetDIBColorTable( HDC hdc, UINT startpos, UINT entries, const RGBQUA
 
     if (!(dc = get_dc_ptr( hdc ))) return 0;
 
-    if ((bitmap = GDI_GetObjPtr( dc->hBitmap, OBJ_BITMAP )))
+    if ((bitmap = GDI_GetObjPtr( dc->hBitmap, NTGDI_OBJ_BITMAP )))
     {
         if (startpos < bitmap->dib.dsBmih.biClrUsed)
         {
@@ -940,10 +936,10 @@ UINT WINAPI SetDIBColorTable( HDC hdc, UINT startpos, UINT entries, const RGBQUA
 
         if (result)  /* update colors of selected objects */
         {
-            SetTextColor( hdc, dc->textColor );
-            SetBkColor( hdc, dc->backgroundColor );
-            SelectObject( hdc, dc->hPen );
-            SelectObject( hdc, dc->hBrush );
+            NtGdiGetAndSetDCDword( hdc, NtGdiSetTextColor, dc->attr->text_color, NULL );
+            NtGdiGetAndSetDCDword( hdc, NtGdiSetBkColor, dc->attr->background_color, NULL );
+            NtGdiSelectPen( hdc, dc->hPen );
+            NtGdiSelectBrush( hdc, dc->hBrush );
         }
     }
     release_dc_ptr( dc );
@@ -951,10 +947,7 @@ UINT WINAPI SetDIBColorTable( HDC hdc, UINT startpos, UINT entries, const RGBQUA
 }
 
 
-/***********************************************************************
- *           GetDIBColorTable    (GDI32.@)
- */
-UINT WINAPI GetDIBColorTable( HDC hdc, UINT startpos, UINT entries, RGBQUAD *colors )
+UINT get_dib_dc_color_table( HDC hdc, UINT startpos, UINT entries, RGBQUAD *colors )
 {
     DC * dc;
     BITMAPOBJ *bitmap;
@@ -962,7 +955,7 @@ UINT WINAPI GetDIBColorTable( HDC hdc, UINT startpos, UINT entries, RGBQUAD *col
 
     if (!(dc = get_dc_ptr( hdc ))) return 0;
 
-    if ((bitmap = GDI_GetObjPtr( dc->hBitmap, OBJ_BITMAP )))
+    if ((bitmap = GDI_GetObjPtr( dc->hBitmap, NTGDI_OBJ_BITMAP )))
     {
         if (startpos < bitmap->dib.dsBmih.biClrUsed)
         {
@@ -1198,22 +1191,13 @@ BITMAPINFO *copy_packed_dib( const BITMAPINFO *src_info, UINT usage )
 }
 
 /******************************************************************************
- * GetDIBits [GDI32.@]
+ *           NtGdiGetDIBitsInternal    (win32u.@)
  *
  * Retrieves bits of bitmap and copies to buffer.
- *
- * RETURNS
- *    Success: Number of scan lines copied from bitmap
- *    Failure: 0
  */
-INT WINAPI DECLSPEC_HOTPATCH GetDIBits(
-    HDC hdc,         /* [in]  Handle to device context */
-    HBITMAP hbitmap, /* [in]  Handle to bitmap */
-    UINT startscan,  /* [in]  First scan line to set in dest bitmap */
-    UINT lines,      /* [in]  Number of scan lines to copy */
-    LPVOID bits,       /* [out] Address of array for bitmap bits */
-    BITMAPINFO * info, /* [out] Address of structure with bitmap data */
-    UINT coloruse)   /* [in]  RGB or palette index */
+INT WINAPI NtGdiGetDIBitsInternal( HDC hdc, HBITMAP hbitmap, UINT startscan, UINT lines,
+                                   void *bits, BITMAPINFO *info, UINT coloruse,
+                                   UINT max_bits, UINT max_info )
 {
     DC * dc;
     BITMAPOBJ * bmp;
@@ -1243,7 +1227,7 @@ INT WINAPI DECLSPEC_HOTPATCH GetDIBits(
         return 0;
     }
     update_dc( dc );
-    if (!(bmp = GDI_GetObjPtr( hbitmap, OBJ_BITMAP )))
+    if (!(bmp = GDI_GetObjPtr( hbitmap, NTGDI_OBJ_BITMAP )))
     {
         release_dc_ptr( dc );
 	return 0;
@@ -1435,35 +1419,30 @@ done:
 
 
 /***********************************************************************
- *           CreateDIBitmap    (GDI32.@)
+ *           NtGdiCreateDIBitmapInternal    (win32u.@)
  *
  * Creates a DDB (device dependent bitmap) from a DIB.
  * The DDB will have the same color depth as the reference DC.
  */
-HBITMAP WINAPI CreateDIBitmap( HDC hdc, const BITMAPINFOHEADER *header,
-                            DWORD init, LPCVOID bits, const BITMAPINFO *data,
-                            UINT coloruse )
+HBITMAP WINAPI NtGdiCreateDIBitmapInternal( HDC hdc, INT width, INT height, DWORD init,
+                                            const void *bits, const BITMAPINFO *data,
+                                            UINT coloruse, UINT max_info, UINT max_bits,
+                                            ULONG flags, HANDLE xform )
 {
-    BITMAPINFOHEADER info;
     HBITMAP handle;
-    LONG height;
 
-    if (!bitmapinfoheader_from_user_bitmapinfo( &info, header )) return 0;
-    if (info.biCompression == BI_JPEG || info.biCompression == BI_PNG) return 0;
-    if (coloruse > DIB_PAL_COLORS + 1) return 0;
-    if (info.biWidth < 0) return 0;
+    if (coloruse > DIB_PAL_COLORS + 1 || width < 0) return 0;
 
     /* Top-down DIBs have a negative height */
-    height = abs( info.biHeight );
+    height = abs( height );
 
-    TRACE("hdc=%p, header=%p, init=%u, bits=%p, data=%p, coloruse=%u (bitmap: width=%d, height=%d, bpp=%u, compr=%u)\n",
-          hdc, header, init, bits, data, coloruse, info.biWidth, info.biHeight,
-          info.biBitCount, info.biCompression);
+    TRACE( "hdc=%p, init=%u, bits=%p, data=%p, coloruse=%u (bitmap: width=%d, height=%d)\n",
+           hdc, init, bits, data, coloruse, width, height );
 
     if (hdc == NULL)
-        handle = CreateBitmap( info.biWidth, height, 1, 1, NULL );
+        handle = NtGdiCreateBitmap( width, height, 1, 1, NULL );
     else
-        handle = CreateCompatibleBitmap( hdc, info.biWidth, height );
+        handle = NtGdiCreateCompatibleBitmap( hdc, width, height );
 
     if (handle)
     {
@@ -1471,7 +1450,7 @@ HBITMAP WINAPI CreateDIBitmap( HDC hdc, const BITMAPINFOHEADER *header,
         {
             if (SetDIBits( hdc, handle, 0, height, bits, data, coloruse ) == 0)
             {
-                DeleteObject( handle );
+                NtGdiDeleteObjectApp( handle );
                 handle = 0;
             }
         }
@@ -1482,10 +1461,11 @@ HBITMAP WINAPI CreateDIBitmap( HDC hdc, const BITMAPINFOHEADER *header,
 
 
 /***********************************************************************
- *           CreateDIBSection    (GDI32.@)
+ *           NtGdiCreateDIBSection    (win32u.@)
  */
-HBITMAP WINAPI DECLSPEC_HOTPATCH CreateDIBSection(HDC hdc, const BITMAPINFO *bmi, UINT usage,
-                                                  void **bits, HANDLE section, DWORD offset)
+HBITMAP WINAPI NtGdiCreateDIBSection( HDC hdc, HANDLE section, DWORD offset, const BITMAPINFO *bmi,
+                                      UINT usage, UINT header_size, ULONG flags,
+                                      ULONG_PTR color_space, void **bits )
 {
     char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
     BITMAPINFO *info = (BITMAPINFO *)buffer;
@@ -1551,35 +1531,40 @@ HBITMAP WINAPI DECLSPEC_HOTPATCH CreateDIBSection(HDC hdc, const BITMAPINFO *bmi
 
     if (section)
     {
-        SYSTEM_INFO SystemInfo;
-        DWORD mapOffset;
-        INT mapSize;
+        LARGE_INTEGER map_offset;
+        SIZE_T map_size;
 
-        GetSystemInfo( &SystemInfo );
-        mapOffset = offset - (offset % SystemInfo.dwAllocationGranularity);
-        mapSize = bmp->dib.dsBmih.biSizeImage + (offset - mapOffset);
-        mapBits = MapViewOfFile( section, FILE_MAP_ALL_ACCESS, 0, mapOffset, mapSize );
-        if (mapBits) bmp->dib.dsBm.bmBits = (char *)mapBits + (offset - mapOffset);
+        map_offset.QuadPart = offset - (offset % system_info.AllocationGranularity);
+        map_size = bmp->dib.dsBmih.biSizeImage + (offset - map_offset.QuadPart);
+        if (NtMapViewOfSection( section, GetCurrentProcess(), &mapBits, 0, 0, &map_offset,
+                                &map_size, ViewShare, 0, PAGE_READWRITE ))
+            goto error;
+        bmp->dib.dsBm.bmBits = (char *)mapBits + (offset - map_offset.QuadPart);
     }
     else
     {
+        SIZE_T size = bmp->dib.dsBmih.biSizeImage;
         offset = 0;
-        bmp->dib.dsBm.bmBits = VirtualAlloc( NULL, bmp->dib.dsBmih.biSizeImage,
-                                             MEM_RESERVE|MEM_COMMIT, PAGE_READWRITE );
+        if (NtAllocateVirtualMemory( GetCurrentProcess(), &bmp->dib.dsBm.bmBits, 0, &size,
+                                     MEM_RESERVE|MEM_COMMIT, PAGE_READWRITE ))
+            goto error;
     }
     bmp->dib.dshSection = section;
     bmp->dib.dsOffset = offset;
 
-    if (!bmp->dib.dsBm.bmBits) goto error;
-
-    if (!(ret = alloc_gdi_handle( bmp, OBJ_BITMAP, &dib_funcs ))) goto error;
-
-    if (bits) *bits = bmp->dib.dsBm.bmBits;
-    return ret;
+    if ((ret = alloc_gdi_handle( &bmp->obj, NTGDI_OBJ_BITMAP, &dib_funcs )))
+    {
+        if (bits) *bits = bmp->dib.dsBm.bmBits;
+        return ret;
+    }
 
+    if (section) NtUnmapViewOfSection( GetCurrentProcess(), mapBits );
+    else
+    {
+        SIZE_T size = 0;
+        NtFreeVirtualMemory( GetCurrentProcess(), &bmp->dib.dsBm.bmBits, &size, MEM_RELEASE );
+    }
 error:
-    if (section) UnmapViewOfFile( mapBits );
-    else VirtualFree( bmp->dib.dsBm.bmBits, 0, MEM_RELEASE );
     HeapFree( GetProcessHeap(), 0, bmp->color_table );
     HeapFree( GetProcessHeap(), 0, bmp );
     return 0;
@@ -1587,9 +1572,9 @@ error:
 
 
 /***********************************************************************
- *           D3DKMTCreateDCFromMemory    (GDI32.@)
+ *           NtGdiDdDDICreateDCFromMemory    (win32u.@)
  */
-NTSTATUS WINAPI D3DKMTCreateDCFromMemory( D3DKMT_CREATEDCFROMMEMORY *desc )
+NTSTATUS WINAPI NtGdiDdDDICreateDCFromMemory( D3DKMT_CREATEDCFROMMEMORY *desc )
 {
     const struct d3dddi_format_info
     {
@@ -1639,7 +1624,8 @@ NTSTATUS WINAPI D3DKMTCreateDCFromMemory( D3DKMT_CREATEDCFROMMEMORY *desc )
         !desc->Pitch || desc->Pitch < get_dib_stride( desc->Width, format->bit_count ) ||
         !desc->Height || desc->Height > UINT_MAX / desc->Pitch) return STATUS_INVALID_PARAMETER;
 
-    if (!desc->hDeviceDc || !(dc = CreateCompatibleDC( desc->hDeviceDc ))) return STATUS_INVALID_PARAMETER;
+    if (!desc->hDeviceDc || !(dc = NtGdiCreateCompatibleDC( desc->hDeviceDc )))
+        return STATUS_INVALID_PARAMETER;
 
     if (!(bmp = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*bmp) ))) goto error;
 
@@ -1684,107 +1670,46 @@ NTSTATUS WINAPI D3DKMTCreateDCFromMemory( D3DKMT_CREATEDCFROMMEMORY *desc )
         }
     }
 
-    if (!(bitmap = alloc_gdi_handle( bmp, OBJ_BITMAP, &dib_funcs ))) goto error;
+    if (!(bitmap = alloc_gdi_handle( &bmp->obj, NTGDI_OBJ_BITMAP, &dib_funcs ))) goto error;
 
     desc->hDc = dc;
     desc->hBitmap = bitmap;
-    SelectObject( dc, bitmap );
+    NtGdiSelectBitmap( dc, bitmap );
     return STATUS_SUCCESS;
 
 error:
     if (bmp) HeapFree( GetProcessHeap(), 0, bmp->color_table );
     HeapFree( GetProcessHeap(), 0, bmp );
-    DeleteDC( dc );
+    NtGdiDeleteObjectApp( dc );
     return STATUS_INVALID_PARAMETER;
 }
 
 
 /***********************************************************************
- *           D3DKMTDestroyDCFromMemory    (GDI32.@)
+ *           NtGdiDdDDIDestroyDCFromMemory    (win32u.@)
  */
-NTSTATUS WINAPI D3DKMTDestroyDCFromMemory( const D3DKMT_DESTROYDCFROMMEMORY *desc )
+NTSTATUS WINAPI NtGdiDdDDIDestroyDCFromMemory( const D3DKMT_DESTROYDCFROMMEMORY *desc )
 {
     if (!desc) return STATUS_INVALID_PARAMETER;
 
     TRACE("dc %p, bitmap %p.\n", desc->hDc, desc->hBitmap);
 
-    if (GetObjectType( desc->hDc ) != OBJ_MEMDC ||
-        GetObjectType( desc->hBitmap ) != OBJ_BITMAP) return STATUS_INVALID_PARAMETER;
-    DeleteObject( desc->hBitmap );
-    DeleteDC( desc->hDc );
+    if (get_gdi_object_type( desc->hDc ) != NTGDI_OBJ_MEMDC ||
+        get_gdi_object_type( desc->hBitmap ) != NTGDI_OBJ_BITMAP) return STATUS_INVALID_PARAMETER;
+    NtGdiDeleteObjectApp( desc->hBitmap );
+    NtGdiDeleteObjectApp( desc->hDc );
 
     return STATUS_SUCCESS;
 }
 
 
-/***********************************************************************
- *           DIB_SelectObject
- */
-static HGDIOBJ DIB_SelectObject( HGDIOBJ handle, HDC hdc )
-{
-    HGDIOBJ ret;
-    BITMAPOBJ *bitmap;
-    DC *dc;
-    PHYSDEV physdev;
-
-    if (!(dc = get_dc_ptr( hdc ))) return 0;
-
-    if (GetObjectType( hdc ) != OBJ_MEMDC)
-    {
-        ret = 0;
-        goto done;
-    }
-    ret = dc->hBitmap;
-    if (handle == dc->hBitmap) goto done;  /* nothing to do */
-
-    if (!(bitmap = GDI_GetObjPtr( handle, OBJ_BITMAP )))
-    {
-        ret = 0;
-        goto done;
-    }
-
-    if (GDI_get_ref_count( handle ))
-    {
-        WARN( "Bitmap already selected in another DC\n" );
-        GDI_ReleaseObj( handle );
-        ret = 0;
-        goto done;
-    }
-
-    physdev = GET_DC_PHYSDEV( dc, pSelectBitmap );
-    if (!physdev->funcs->pSelectBitmap( physdev, handle ))
-    {
-        GDI_ReleaseObj( handle );
-        ret = 0;
-    }
-    else
-    {
-        dc->hBitmap = handle;
-        GDI_inc_ref_count( handle );
-        dc->dirty = 0;
-        dc->vis_rect.left   = 0;
-        dc->vis_rect.top    = 0;
-        dc->vis_rect.right  = bitmap->dib.dsBm.bmWidth;
-        dc->vis_rect.bottom = bitmap->dib.dsBm.bmHeight;
-        dc->device_rect = dc->vis_rect;
-        GDI_ReleaseObj( handle );
-        DC_InitDC( dc );
-        GDI_dec_ref_count( ret );
-    }
-
- done:
-    release_dc_ptr( dc );
-    return ret;
-}
-
-
 /***********************************************************************
  *           DIB_GetObject
  */
 static INT DIB_GetObject( HGDIOBJ handle, INT count, LPVOID buffer )
 {
     INT ret = 0;
-    BITMAPOBJ *bmp = GDI_GetObjPtr( handle, OBJ_BITMAP );
+    BITMAPOBJ *bmp = GDI_GetObjPtr( handle, NTGDI_OBJ_BITMAP );
 
     if (!bmp) return 0;
 
@@ -1821,12 +1746,14 @@ static BOOL DIB_DeleteObject( HGDIOBJ handle )
 
     if (bmp->dib.dshSection)
     {
-        SYSTEM_INFO SystemInfo;
-        GetSystemInfo( &SystemInfo );
-        UnmapViewOfFile( (char *)bmp->dib.dsBm.bmBits -
-                         (bmp->dib.dsOffset % SystemInfo.dwAllocationGranularity) );
+        NtUnmapViewOfSection( GetCurrentProcess(), (char *)bmp->dib.dsBm.bmBits -
+                              (bmp->dib.dsOffset % system_info.AllocationGranularity) );
+    }
+    else
+    {
+        SIZE_T size = 0;
+        NtFreeVirtualMemory( GetCurrentProcess(), &bmp->dib.dsBm.bmBits, &size, MEM_RELEASE );
     }
-    else VirtualFree( bmp->dib.dsBm.bmBits, 0, MEM_RELEASE );
 
     HeapFree(GetProcessHeap(), 0, bmp->color_table);
     HeapFree( GetProcessHeap(), 0, bmp );
