@@ -31,57 +31,44 @@
 #include "winerror.h"
 #include "winternl.h"
 
-#include "gdi_private.h"
+#include "ntgdi_private.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(gdi);
 
 #define FIRST_GDI_HANDLE 32
-#define MAX_GDI_HANDLES  16384
 
-struct hdc_list
-{
-    HDC hdc;
-    struct hdc_list *next;
-};
-
-struct gdi_handle_entry
-{
-    void                       *obj;         /* pointer to the object-specific data */
-    const struct gdi_obj_funcs *funcs;       /* type-specific functions */
-    struct hdc_list            *hdcs;        /* list of HDCs interested in this object */
-    WORD                        generation;  /* generation count for reusing handle values */
-    WORD                        type;        /* object type (one of the OBJ_* constants) */
-    WORD                        selcount;    /* number of times the object is selected in a DC */
-    WORD                        system : 1;  /* system object flag */
-    WORD                        deleted : 1; /* whether DeleteObject has been called on this object */
-};
-
-static struct gdi_handle_entry gdi_handles[MAX_GDI_HANDLES];
-static struct gdi_handle_entry *next_free;
-static struct gdi_handle_entry *next_unused = gdi_handles;
+static GDI_SHARED_MEMORY gdi_shared;
+static GDI_HANDLE_ENTRY *next_free;
+static GDI_HANDLE_ENTRY *next_unused = gdi_shared.Handles + FIRST_GDI_HANDLE;
 static LONG debug_count;
 HMODULE gdi32_module = 0;
+SYSTEM_BASIC_INFORMATION system_info;
 
-static inline HGDIOBJ entry_to_handle( struct gdi_handle_entry *entry )
+static inline HGDIOBJ entry_to_handle( GDI_HANDLE_ENTRY *entry )
 {
-    unsigned int idx = entry - gdi_handles + FIRST_GDI_HANDLE;
-    return LongToHandle( idx | (entry->generation << 16) );
+    unsigned int idx = entry - gdi_shared.Handles;
+    return LongToHandle( idx | (entry->Unique << NTGDI_HANDLE_TYPE_SHIFT) );
 }
 
-static inline struct gdi_handle_entry *handle_entry( HGDIOBJ handle )
+static inline GDI_HANDLE_ENTRY *handle_entry( HGDIOBJ handle )
 {
-    unsigned int idx = LOWORD(handle) - FIRST_GDI_HANDLE;
+    unsigned int idx = LOWORD(handle);
 
-    if (idx < MAX_GDI_HANDLES && gdi_handles[idx].type)
+    if (idx < GDI_MAX_HANDLE_COUNT && gdi_shared.Handles[idx].Type)
     {
-        if (!HIWORD( handle ) || HIWORD( handle ) == gdi_handles[idx].generation)
-            return &gdi_handles[idx];
+        if (!HIWORD( handle ) || HIWORD( handle ) == gdi_shared.Handles[idx].Unique)
+            return &gdi_shared.Handles[idx];
     }
     if (handle) WARN( "invalid handle %p\n", handle );
     return NULL;
 }
 
+static inline struct gdi_obj_header *entry_obj( GDI_HANDLE_ENTRY *entry )
+{
+    return (struct gdi_obj_header *)(ULONG_PTR)entry->Object;
+}
+
 /***********************************************************************
  *          GDI stock objects
  */
@@ -94,19 +81,7 @@ static const LOGBRUSH LtGrayBrush = { BS_SOLID, RGB(192,192,192), 0 };
 static const LOGBRUSH GrayBrush   = { BS_SOLID, RGB(128,128,128), 0 };
 static const LOGBRUSH DkGrayBrush = { BS_SOLID, RGB(64,64,64), 0 };
 
-static const LOGPEN WhitePen = { PS_SOLID, { 0, 0 }, RGB(255,255,255) };
-static const LOGPEN BlackPen = { PS_SOLID, { 0, 0 }, RGB(0,0,0) };
-static const LOGPEN NullPen  = { PS_NULL,  { 0, 0 }, 0 };
-
 static const LOGBRUSH DCBrush = { BS_SOLID, RGB(255,255,255), 0 };
-static const LOGPEN DCPen     = { PS_SOLID, { 0, 0 }, RGB(0,0,0) };
-
-/* reserve one extra entry for the stock default bitmap */
-/* this is what Windows does too */
-#define NB_STOCK_OBJECTS (STOCK_LAST+2)
-
-static HGDIOBJ stock_objects[NB_STOCK_OBJECTS];
-static HGDIOBJ scaled_stock_objects[NB_STOCK_OBJECTS];
 
 static CRITICAL_SECTION gdi_section;
 static CRITICAL_SECTION_DEBUG critsect_debug =
@@ -421,7 +396,7 @@ static const struct DefaultFontInfo default_fonts[] =
 
 
 /*************************************************************************
- * __wine_make_gdi_object_system    (GDI32.@)
+ * __wine_make_gdi_object_system    (win32u.@)
  *
  * USER has to tell GDI that its system brushes and pens are non-deletable.
  * For a description of the GDI object magics and their flags,
@@ -429,10 +404,10 @@ static const struct DefaultFontInfo default_fonts[] =
  */
 void CDECL __wine_make_gdi_object_system( HGDIOBJ handle, BOOL set)
 {
-    struct gdi_handle_entry *entry;
+    GDI_HANDLE_ENTRY *entry;
 
     EnterCriticalSection( &gdi_section );
-    if ((entry = handle_entry( handle ))) entry->system = !!set;
+    if ((entry = handle_entry( handle ))) entry_obj( entry )->system = !!set;
     LeaveCriticalSection( &gdi_section );
 }
 
@@ -466,7 +441,7 @@ static UINT get_default_charset( void )
 
     uACP = GetACP();
     csi.ciCharset = ANSI_CHARSET;
-    if ( !TranslateCharsetInfo( ULongToPtr(uACP), &csi, TCI_SRCCODEPAGE ) )
+    if ( !translate_charset_info( ULongToPtr(uACP), &csi, TCI_SRCCODEPAGE ) )
     {
         FIXME( "unhandled codepage %u - use ANSI_CHARSET for default stock objects\n", uACP );
         return ANSI_CHARSET;
@@ -484,11 +459,11 @@ static UINT get_default_charset( void )
  */
 UINT GDI_get_ref_count( HGDIOBJ handle )
 {
-    struct gdi_handle_entry *entry;
+    GDI_HANDLE_ENTRY *entry;
     UINT ret = 0;
 
     EnterCriticalSection( &gdi_section );
-    if ((entry = handle_entry( handle ))) ret = entry->selcount;
+    if ((entry = handle_entry( handle ))) ret = entry_obj( entry )->selcount;
     LeaveCriticalSection( &gdi_section );
     return ret;
 }
@@ -501,10 +476,10 @@ UINT GDI_get_ref_count( HGDIOBJ handle )
  */
 HGDIOBJ GDI_inc_ref_count( HGDIOBJ handle )
 {
-    struct gdi_handle_entry *entry;
+    GDI_HANDLE_ENTRY *entry;
 
     EnterCriticalSection( &gdi_section );
-    if ((entry = handle_entry( handle ))) entry->selcount++;
+    if ((entry = handle_entry( handle ))) entry_obj( entry )->selcount++;
     else handle = 0;
     LeaveCriticalSection( &gdi_section );
     return handle;
@@ -518,19 +493,19 @@ HGDIOBJ GDI_inc_ref_count( HGDIOBJ handle )
  */
 BOOL GDI_dec_ref_count( HGDIOBJ handle )
 {
-    struct gdi_handle_entry *entry;
+    GDI_HANDLE_ENTRY *entry;
 
     EnterCriticalSection( &gdi_section );
     if ((entry = handle_entry( handle )))
     {
-        assert( entry->selcount );
-        if (!--entry->selcount && entry->deleted)
+        assert( entry_obj( entry )->selcount );
+        if (!--entry_obj( entry )->selcount && entry_obj( entry )->deleted)
         {
             /* handle delayed DeleteObject*/
-            entry->deleted = 0;
+            entry_obj( entry )->deleted = 0;
             LeaveCriticalSection( &gdi_section );
             TRACE( "executing delayed DeleteObject for %p\n", handle );
-            DeleteObject( handle );
+            NtGdiDeleteObjectApp( handle );
             return TRUE;
         }
     }
@@ -596,6 +571,15 @@ DWORD get_system_dpi(void)
     return pGetDpiForSystem ? pGetDpiForSystem() : 96;
 }
 
+static HFONT create_font( const LOGFONTW *deffont )
+{
+    ENUMLOGFONTEXDVW lf;
+
+    memset( &lf, 0, sizeof(lf) );
+    lf.elfEnumLogfontEx.elfLogFont = *deffont;
+    return NtGdiHfontCreate( &lf, sizeof(lf), 0, 0, NULL );
+}
+
 static HFONT create_scaled_font( const LOGFONTW *deffont )
 {
     LOGFONTW lf;
@@ -608,8 +592,104 @@ static HFONT create_scaled_font( const LOGFONTW *deffont )
     }
 
     lf = *deffont;
-    lf.lfHeight = MulDiv( lf.lfHeight, dpi, 96 );
-    return CreateFontIndirectW( &lf );
+    lf.lfHeight = muldiv( lf.lfHeight, dpi, 96 );
+    return create_font( &lf );
+}
+
+static void set_gdi_shared(void)
+{
+#ifndef _WIN64
+    if (NtCurrentTeb()->GdiBatchCount)
+    {
+        TEB64 *teb64 = (TEB64 *)(UINT_PTR)NtCurrentTeb()->GdiBatchCount;
+        PEB64 *peb64 = (PEB64 *)(UINT_PTR)teb64->Peb;
+        peb64->GdiSharedHandleTable = (UINT_PTR)&gdi_shared;
+        return;
+    }
+#endif
+    /* NOTE: Windows uses 32-bit for 32-bit kernel */
+    NtCurrentTeb()->Peb->GdiSharedHandleTable = &gdi_shared;
+}
+
+HGDIOBJ get_stock_object( INT obj )
+{
+    assert( obj >= 0 && obj <= STOCK_LAST + 1 && obj != 9 );
+
+    switch (obj)
+    {
+    case OEM_FIXED_FONT:
+        if (get_system_dpi() != 96) obj = 9;
+        break;
+    case SYSTEM_FONT:
+        if (get_system_dpi() != 96) obj = STOCK_LAST + 2;
+        break;
+    case SYSTEM_FIXED_FONT:
+        if (get_system_dpi() != 96) obj = STOCK_LAST + 3;
+        break;
+    case DEFAULT_GUI_FONT:
+        if (get_system_dpi() != 96) obj = STOCK_LAST + 4;
+        break;
+    }
+
+    return entry_to_handle( handle_entry( ULongToHandle( obj + FIRST_GDI_HANDLE )));
+}
+
+static void init_stock_objects(void)
+{
+    const struct DefaultFontInfo *deffonts;
+    unsigned int i;
+    HGDIOBJ obj;
+
+    /* Create stock objects in order matching stock object macros,
+     * so that they use predictable handle slots. Our GetStockObject
+     * depends on it. */
+    create_brush( &WhiteBrush );
+    create_brush( &LtGrayBrush );
+    create_brush( &GrayBrush );
+    create_brush( &DkGrayBrush );
+    create_brush( &BlackBrush );
+    create_brush( &NullBrush );
+
+    create_pen( PS_SOLID, 0, RGB(255,255,255) );
+    create_pen( PS_SOLID, 0, RGB(0,0,0) );
+    create_pen( PS_NULL,  0, RGB(0,0,0) );
+
+    /* slot 9 is not used for non-scaled stock objects */
+    create_scaled_font( &OEMFixedFont );
+
+    /* language-independent stock fonts */
+    create_font( &OEMFixedFont );
+    create_font( &AnsiFixedFont );
+    create_font( &AnsiVarFont );
+
+    /* language-dependent stock fonts */
+    deffonts = get_default_fonts(get_default_charset());
+    create_font( &deffonts->SystemFont );
+    create_font( &deffonts->DeviceDefaultFont );
+
+    PALETTE_Init();
+
+    create_font( &deffonts->SystemFixedFont );
+    create_font( &deffonts->DefaultGuiFont );
+
+    create_brush( &DCBrush );
+    NtGdiCreatePen( PS_SOLID, 0, RGB(0,0,0), NULL );
+
+    obj = NtGdiCreateBitmap( 1, 1, 1, 1, NULL );
+
+    assert( (HandleToULong( obj ) & 0xffff) == FIRST_GDI_HANDLE + DEFAULT_BITMAP );
+
+    create_scaled_font( &deffonts->SystemFont );
+    create_scaled_font( &deffonts->SystemFixedFont );
+    create_scaled_font( &deffonts->DefaultGuiFont );
+
+    /* clear the NOSYSTEM bit on all stock objects*/
+    for (i = 0; i < STOCK_LAST + 5; i++)
+    {
+        GDI_HANDLE_ENTRY *entry = &gdi_shared.Handles[FIRST_GDI_HANDLE + i];
+        entry_obj( entry )->system = TRUE;
+        entry->StockFlag = 1;
+    }
 }
 
 /***********************************************************************
@@ -619,68 +699,14 @@ static HFONT create_scaled_font( const LOGFONTW *deffont )
  */
 BOOL WINAPI DllMain( HINSTANCE inst, DWORD reason, LPVOID reserved )
 {
-    const struct DefaultFontInfo* deffonts;
-    int i;
-
     if (reason != DLL_PROCESS_ATTACH) return TRUE;
 
     gdi32_module = inst;
     DisableThreadLibraryCalls( inst );
+    NtQuerySystemInformation( SystemBasicInformation, &system_info, sizeof(system_info), NULL );
+    set_gdi_shared();
     font_init();
-
-    /* create stock objects */
-    stock_objects[WHITE_BRUSH]  = CreateBrushIndirect( &WhiteBrush );
-    stock_objects[LTGRAY_BRUSH] = CreateBrushIndirect( &LtGrayBrush );
-    stock_objects[GRAY_BRUSH]   = CreateBrushIndirect( &GrayBrush );
-    stock_objects[DKGRAY_BRUSH] = CreateBrushIndirect( &DkGrayBrush );
-    stock_objects[BLACK_BRUSH]  = CreateBrushIndirect( &BlackBrush );
-    stock_objects[NULL_BRUSH]   = CreateBrushIndirect( &NullBrush );
-
-    stock_objects[WHITE_PEN]    = CreatePenIndirect( &WhitePen );
-    stock_objects[BLACK_PEN]    = CreatePenIndirect( &BlackPen );
-    stock_objects[NULL_PEN]     = CreatePenIndirect( &NullPen );
-
-    stock_objects[DEFAULT_PALETTE] = PALETTE_Init();
-    stock_objects[DEFAULT_BITMAP]  = CreateBitmap( 1, 1, 1, 1, NULL );
-
-    /* language-independent stock fonts */
-    stock_objects[OEM_FIXED_FONT]      = CreateFontIndirectW( &OEMFixedFont );
-    stock_objects[ANSI_FIXED_FONT]     = CreateFontIndirectW( &AnsiFixedFont );
-    stock_objects[ANSI_VAR_FONT]       = CreateFontIndirectW( &AnsiVarFont );
-
-    /* language-dependent stock fonts */
-    deffonts = get_default_fonts(get_default_charset());
-    stock_objects[SYSTEM_FONT]         = CreateFontIndirectW( &deffonts->SystemFont );
-    stock_objects[DEVICE_DEFAULT_FONT] = CreateFontIndirectW( &deffonts->DeviceDefaultFont );
-    stock_objects[SYSTEM_FIXED_FONT]   = CreateFontIndirectW( &deffonts->SystemFixedFont );
-    stock_objects[DEFAULT_GUI_FONT]    = CreateFontIndirectW( &deffonts->DefaultGuiFont );
-
-    scaled_stock_objects[OEM_FIXED_FONT]    = create_scaled_font( &OEMFixedFont );
-    scaled_stock_objects[SYSTEM_FONT]       = create_scaled_font( &deffonts->SystemFont );
-    scaled_stock_objects[SYSTEM_FIXED_FONT] = create_scaled_font( &deffonts->SystemFixedFont );
-    scaled_stock_objects[DEFAULT_GUI_FONT]  = create_scaled_font( &deffonts->DefaultGuiFont );
-
-    stock_objects[DC_BRUSH]     = CreateBrushIndirect( &DCBrush );
-    stock_objects[DC_PEN]       = CreatePenIndirect( &DCPen );
-
-    /* clear the NOSYSTEM bit on all stock objects*/
-    for (i = 0; i < NB_STOCK_OBJECTS; i++)
-    {
-        if (stock_objects[i]) __wine_make_gdi_object_system( stock_objects[i], TRUE );
-        if (scaled_stock_objects[i]) __wine_make_gdi_object_system( scaled_stock_objects[i], TRUE );
-    }
-    return TRUE;
-}
-
-
-/***********************************************************************
- *           GdiDllInitialize
- *
- * Stub entry point, some games (CoD: Black Ops 3) call it directly.
- */
-BOOL WINAPI GdiDllInitialize( HINSTANCE inst, DWORD reason, LPVOID reserved )
-{
-    FIXME("stub\n");
+    init_stock_objects();
     return TRUE;
 }
 
@@ -689,39 +715,39 @@ static const char *gdi_obj_type( unsigned type )
 {
     switch ( type )
     {
-        case OBJ_PEN: return "OBJ_PEN";
-        case OBJ_BRUSH: return "OBJ_BRUSH";
-        case OBJ_DC: return "OBJ_DC";
-        case OBJ_METADC: return "OBJ_METADC";
-        case OBJ_PAL: return "OBJ_PAL";
-        case OBJ_FONT: return "OBJ_FONT";
-        case OBJ_BITMAP: return "OBJ_BITMAP";
-        case OBJ_REGION: return "OBJ_REGION";
-        case OBJ_METAFILE: return "OBJ_METAFILE";
-        case OBJ_MEMDC: return "OBJ_MEMDC";
-        case OBJ_EXTPEN: return "OBJ_EXTPEN";
-        case OBJ_ENHMETADC: return "OBJ_ENHMETADC";
-        case OBJ_ENHMETAFILE: return "OBJ_ENHMETAFILE";
-        case OBJ_COLORSPACE: return "OBJ_COLORSPACE";
+        case NTGDI_OBJ_PEN: return "NTGDI_OBJ_PEN";
+        case NTGDI_OBJ_BRUSH: return "NTGDI_OBJ_BRUSH";
+        case NTGDI_OBJ_DC: return "NTGDI_OBJ_DC";
+        case NTGDI_OBJ_METADC: return "NTGDI_OBJ_METADC";
+        case NTGDI_OBJ_PAL: return "NTGDI_OBJ_PAL";
+        case NTGDI_OBJ_FONT: return "NTGDI_OBJ_FONT";
+        case NTGDI_OBJ_BITMAP: return "NTGDI_OBJ_BITMAP";
+        case NTGDI_OBJ_REGION: return "NTGDI_OBJ_REGION";
+        case NTGDI_OBJ_METAFILE: return "NTGDI_OBJ_METAFILE";
+        case NTGDI_OBJ_MEMDC: return "NTGDI_OBJ_MEMDC";
+        case NTGDI_OBJ_EXTPEN: return "NTGDI_OBJ_EXTPEN";
+        case NTGDI_OBJ_ENHMETADC: return "NTGDI_OBJ_ENHMETADC";
+        case NTGDI_OBJ_ENHMETAFILE: return "NTGDI_OBJ_ENHMETAFILE";
         default: return "UNKNOWN";
     }
 }
 
 static void dump_gdi_objects( void )
 {
-    struct gdi_handle_entry *entry;
+    GDI_HANDLE_ENTRY *entry;
 
-    TRACE( "%u objects:\n", MAX_GDI_HANDLES );
+    TRACE( "%u objects:\n", GDI_MAX_HANDLE_COUNT );
 
     EnterCriticalSection( &gdi_section );
-    for (entry = gdi_handles; entry < next_unused; entry++)
+    for (entry = gdi_shared.Handles; entry < next_unused; entry++)
     {
-        if (!entry->type)
+        if (!entry->Type)
             TRACE( "handle %p FREE\n", entry_to_handle( entry ));
         else
-            TRACE( "handle %p obj %p type %s selcount %u deleted %u\n",
-                   entry_to_handle( entry ), entry->obj, gdi_obj_type( entry->type ),
-                   entry->selcount, entry->deleted );
+            TRACE( "handle %p obj %s type %s selcount %u deleted %u\n",
+                   entry_to_handle( entry ), wine_dbgstr_longlong( entry->Object ),
+                   gdi_obj_type( entry->ExtType << NTGDI_HANDLE_TYPE_SHIFT ),
+                   entry_obj( entry )->selcount, entry_obj( entry )->deleted );
     }
     LeaveCriticalSection( &gdi_section );
 }
@@ -731,9 +757,9 @@ static void dump_gdi_objects( void )
  *
  * Allocate a GDI handle for an object, which must have been allocated on the process heap.
  */
-HGDIOBJ alloc_gdi_handle( void *obj, WORD type, const struct gdi_obj_funcs *funcs )
+HGDIOBJ alloc_gdi_handle( struct gdi_obj_header *obj, DWORD type, const struct gdi_obj_funcs *funcs )
 {
-    struct gdi_handle_entry *entry;
+    GDI_HANDLE_ENTRY *entry;
     HGDIOBJ ret;
 
     assert( type );  /* type 0 is reserved to mark free entries */
@@ -742,8 +768,8 @@ HGDIOBJ alloc_gdi_handle( void *obj, WORD type, const struct gdi_obj_funcs *func
 
     entry = next_free;
     if (entry)
-        next_free = entry->obj;
-    else if (next_unused < gdi_handles + MAX_GDI_HANDLES)
+        next_free = (GDI_HANDLE_ENTRY *)(UINT_PTR)entry->Object;
+    else if (next_unused < gdi_shared.Handles + GDI_MAX_HANDLE_COUNT)
         entry = next_unused++;
     else
     {
@@ -752,18 +778,18 @@ HGDIOBJ alloc_gdi_handle( void *obj, WORD type, const struct gdi_obj_funcs *func
         if (TRACE_ON(gdi)) dump_gdi_objects();
         return 0;
     }
-    entry->obj      = obj;
-    entry->funcs    = funcs;
-    entry->hdcs     = NULL;
-    entry->type     = type;
-    entry->selcount = 0;
-    entry->system   = 0;
-    entry->deleted  = 0;
-    if (++entry->generation == 0xffff) entry->generation = 1;
+    obj->funcs    = funcs;
+    obj->selcount = 0;
+    obj->system   = 0;
+    obj->deleted  = 0;
+    entry->Object  = (UINT_PTR)obj;
+    entry->ExtType = type >> NTGDI_HANDLE_TYPE_SHIFT;
+    entry->Type    = entry->ExtType & 0x1f;
+    if (++entry->Generation == 0xff) entry->Generation = 1;
     ret = entry_to_handle( entry );
     LeaveCriticalSection( &gdi_section );
     TRACE( "allocated %s %p %u/%u\n", gdi_obj_type(type), ret,
-           InterlockedIncrement( &debug_count ), MAX_GDI_HANDLES );
+           InterlockedIncrement( &debug_count ), GDI_MAX_HANDLE_COUNT );
     return ret;
 }
 
@@ -776,39 +802,26 @@ HGDIOBJ alloc_gdi_handle( void *obj, WORD type, const struct gdi_obj_funcs *func
 void *free_gdi_handle( HGDIOBJ handle )
 {
     void *object = NULL;
-    struct gdi_handle_entry *entry;
+    GDI_HANDLE_ENTRY *entry;
 
     EnterCriticalSection( &gdi_section );
     if ((entry = handle_entry( handle )))
     {
-        TRACE( "freed %s %p %u/%u\n", gdi_obj_type( entry->type ), handle,
-               InterlockedDecrement( &debug_count ) + 1, MAX_GDI_HANDLES );
-        object = entry->obj;
-        entry->type = 0;
-        entry->obj = next_free;
+        TRACE( "freed %s %p %u/%u\n", gdi_obj_type( entry->ExtType << NTGDI_HANDLE_TYPE_SHIFT ),
+               handle, InterlockedDecrement( &debug_count ) + 1, GDI_MAX_HANDLE_COUNT );
+        object = entry_obj( entry );
+        entry->Type = 0;
+        entry->Object = (UINT_PTR)next_free;
         next_free = entry;
     }
     LeaveCriticalSection( &gdi_section );
     return object;
 }
 
-
-/***********************************************************************
- *           get_full_gdi_handle
- *
- * Return the full GDI handle from a possibly truncated value.
- */
-HGDIOBJ get_full_gdi_handle( HGDIOBJ handle )
+DWORD get_gdi_object_type( HGDIOBJ obj )
 {
-    struct gdi_handle_entry *entry;
-
-    if (!HIWORD( handle ))
-    {
-        EnterCriticalSection( &gdi_section );
-        if ((entry = handle_entry( handle ))) handle = entry_to_handle( entry );
-        LeaveCriticalSection( &gdi_section );
-    }
-    return handle;
+    GDI_HANDLE_ENTRY *entry = handle_entry( obj );
+    return entry ? entry->ExtType << NTGDI_HANDLE_TYPE_SHIFT : 0;
 }
 
 /***********************************************************************
@@ -818,17 +831,17 @@ HGDIOBJ get_full_gdi_handle( HGDIOBJ handle )
  * associated with the handle.
  * The object must be released with GDI_ReleaseObj.
  */
-void *get_any_obj_ptr( HGDIOBJ handle, WORD *type )
+void *get_any_obj_ptr( HGDIOBJ handle, DWORD *type )
 {
     void *ptr = NULL;
-    struct gdi_handle_entry *entry;
+    GDI_HANDLE_ENTRY *entry;
 
     EnterCriticalSection( &gdi_section );
 
     if ((entry = handle_entry( handle )))
     {
-        ptr = entry->obj;
-        *type = entry->type;
+        ptr = entry_obj( entry );
+        *type = entry->ExtType << NTGDI_HANDLE_TYPE_SHIFT;
     }
 
     if (!ptr) LeaveCriticalSection( &gdi_section );
@@ -842,9 +855,9 @@ void *get_any_obj_ptr( HGDIOBJ handle, WORD *type )
  * Return NULL if the object has the wrong type.
  * The object must be released with GDI_ReleaseObj.
  */
-void *GDI_GetObjPtr( HGDIOBJ handle, WORD type )
+void *GDI_GetObjPtr( HGDIOBJ handle, DWORD type )
 {
-    WORD ret_type;
+    DWORD ret_type;
     void *ptr = get_any_obj_ptr( handle, &ret_type );
     if (ptr && ret_type != type)
     {
@@ -872,13 +885,13 @@ void GDI_CheckNotLock(void)
     if (RtlIsCriticalSectionLockedByThread(&gdi_section))
     {
         ERR( "BUG: holding GDI lock\n" );
-        DebugBreak();
+        assert( 0 );
     }
 }
 
 
 /***********************************************************************
- *           DeleteObject    (GDI32.@)
+ *           NtGdiDeleteObjectApp    (win32u.@)
  *
  * Delete a Gdi object.
  *
@@ -891,11 +904,11 @@ void GDI_CheckNotLock(void)
  *  Failure: FALSE, if obj is not a valid Gdi object, or is currently selected
  *           into a DC.
  */
-BOOL WINAPI DeleteObject( HGDIOBJ obj )
+BOOL WINAPI NtGdiDeleteObjectApp( HGDIOBJ obj )
 {
-    struct gdi_handle_entry *entry;
-    struct hdc_list *hdcs_head;
+    GDI_HANDLE_ENTRY *entry;
     const struct gdi_obj_funcs *funcs = NULL;
+    struct gdi_obj_header *header;
 
     EnterCriticalSection( &gdi_section );
     if (!(entry = handle_entry( obj )))
@@ -904,7 +917,8 @@ BOOL WINAPI DeleteObject( HGDIOBJ obj )
         return FALSE;
     }
 
-    if (entry->system)
+    header = entry_obj( entry );
+    if (header->system)
     {
 	TRACE("Preserving system object %p\n", obj);
         LeaveCriticalSection( &gdi_section );
@@ -913,34 +927,15 @@ BOOL WINAPI DeleteObject( HGDIOBJ obj )
 
     obj = entry_to_handle( entry );  /* make it a full handle */
 
-    hdcs_head = entry->hdcs;
-    entry->hdcs = NULL;
-
-    if (entry->selcount)
+    if (header->selcount)
     {
-        TRACE("delayed for %p because object in use, count %u\n", obj, entry->selcount );
-        entry->deleted = 1;  /* mark for delete */
+        TRACE("delayed for %p because object in use, count %u\n", obj, header->selcount );
+        header->deleted = 1;  /* mark for delete */
     }
-    else funcs = entry->funcs;
+    else funcs = header->funcs;
 
     LeaveCriticalSection( &gdi_section );
 
-    while (hdcs_head)
-    {
-        struct hdc_list *next = hdcs_head->next;
-        DC *dc = get_dc_ptr(hdcs_head->hdc);
-
-        TRACE("hdc %p has interest in %p\n", hdcs_head->hdc, obj);
-        if(dc)
-        {
-            PHYSDEV physdev = GET_DC_PHYSDEV( dc, pDeleteObject );
-            physdev->funcs->pDeleteObject( physdev, obj );
-            release_dc_ptr( dc );
-        }
-        HeapFree(GetProcessHeap(), 0, hdcs_head);
-        hdcs_head = next;
-    }
-
     TRACE("%p\n", obj );
 
     if (funcs && funcs->pDeleteObject) return funcs->pDeleteObject( obj );
@@ -948,85 +943,39 @@ BOOL WINAPI DeleteObject( HGDIOBJ obj )
 }
 
 /***********************************************************************
- *           GDI_hdc_using_object
- *
- * Call this if the dc requires DeleteObject notification
+ *           NtGdiCreateClientObj    (win32u.@)
  */
-void GDI_hdc_using_object(HGDIOBJ obj, HDC hdc)
+HANDLE WINAPI NtGdiCreateClientObj( ULONG type )
 {
-    struct gdi_handle_entry *entry;
-    struct hdc_list *phdc;
+    struct gdi_obj_header *obj;
+    HGDIOBJ handle;
 
-    TRACE("obj %p hdc %p\n", obj, hdc);
+    if (!(obj = HeapAlloc( GetProcessHeap(), 0, sizeof(*obj) )))
+        return 0;
 
-    EnterCriticalSection( &gdi_section );
-    if ((entry = handle_entry( obj )) && !entry->system)
-    {
-        for (phdc = entry->hdcs; phdc; phdc = phdc->next)
-            if (phdc->hdc == hdc) break;
-
-        if (!phdc)
-        {
-            phdc = HeapAlloc(GetProcessHeap(), 0, sizeof(*phdc));
-            phdc->hdc = hdc;
-            phdc->next = entry->hdcs;
-            entry->hdcs = phdc;
-        }
-    }
-    LeaveCriticalSection( &gdi_section );
+    handle = alloc_gdi_handle( obj, type, NULL );
+    if (!handle) HeapFree( GetProcessHeap(), 0, obj );
+    return handle;
 }
 
 /***********************************************************************
- *           GDI_hdc_not_using_object
- *
+ *           NtGdiDeleteClientObj    (win32u.@)
  */
-void GDI_hdc_not_using_object(HGDIOBJ obj, HDC hdc)
+BOOL WINAPI NtGdiDeleteClientObj( HGDIOBJ handle )
 {
-    struct gdi_handle_entry *entry;
-    struct hdc_list **pphdc;
-
-    TRACE("obj %p hdc %p\n", obj, hdc);
-
-    EnterCriticalSection( &gdi_section );
-    if ((entry = handle_entry( obj )) && !entry->system)
-    {
-        for (pphdc = &entry->hdcs; *pphdc; pphdc = &(*pphdc)->next)
-            if ((*pphdc)->hdc == hdc)
-            {
-                struct hdc_list *phdc = *pphdc;
-                *pphdc = phdc->next;
-                HeapFree(GetProcessHeap(), 0, phdc);
-                break;
-            }
-    }
-    LeaveCriticalSection( &gdi_section );
-}
-
-/***********************************************************************
- *           GetStockObject    (GDI32.@)
- */
-HGDIOBJ WINAPI GetStockObject( INT obj )
-{
-    if ((obj < 0) || (obj >= NB_STOCK_OBJECTS)) return 0;
-    switch (obj)
-    {
-    case OEM_FIXED_FONT:
-    case SYSTEM_FONT:
-    case SYSTEM_FIXED_FONT:
-    case DEFAULT_GUI_FONT:
-        if (get_system_dpi() != 96) return scaled_stock_objects[obj];
-        break;
-    }
-    return stock_objects[obj];
+    void *obj;
+    if (!(obj = free_gdi_handle( handle ))) return FALSE;
+    HeapFree( GetProcessHeap(), 0, obj );
+    return TRUE;
 }
 
 
 /***********************************************************************
- *           GetObjectA    (GDI32.@)
+ *           NtGdiExtGetObjectW    (win32u.@)
  */
-INT WINAPI GetObjectA( HGDIOBJ handle, INT count, LPVOID buffer )
+INT WINAPI NtGdiExtGetObjectW( HGDIOBJ handle, INT count, void *buffer )
 {
-    struct gdi_handle_entry *entry;
+    GDI_HANDLE_ENTRY *entry;
     const struct gdi_obj_funcs *funcs = NULL;
     INT result = 0;
 
@@ -1035,47 +984,14 @@ INT WINAPI GetObjectA( HGDIOBJ handle, INT count, LPVOID buffer )
     EnterCriticalSection( &gdi_section );
     if ((entry = handle_entry( handle )))
     {
-        funcs = entry->funcs;
+        funcs = entry_obj( entry )->funcs;
         handle = entry_to_handle( entry );  /* make it a full handle */
     }
     LeaveCriticalSection( &gdi_section );
 
-    if (funcs)
+    if (funcs && funcs->pGetObjectW)
     {
-        if (!funcs->pGetObjectA)
-            SetLastError( ERROR_INVALID_HANDLE );
-        else if (buffer && ((ULONG_PTR)buffer >> 16) == 0) /* catch apps getting argument order wrong */
-            SetLastError( ERROR_NOACCESS );
-        else
-            result = funcs->pGetObjectA( handle, count, buffer );
-    }
-    return result;
-}
-
-/***********************************************************************
- *           GetObjectW    (GDI32.@)
- */
-INT WINAPI GetObjectW( HGDIOBJ handle, INT count, LPVOID buffer )
-{
-    struct gdi_handle_entry *entry;
-    const struct gdi_obj_funcs *funcs = NULL;
-    INT result = 0;
-
-    TRACE("%p %d %p\n", handle, count, buffer );
-
-    EnterCriticalSection( &gdi_section );
-    if ((entry = handle_entry( handle )))
-    {
-        funcs = entry->funcs;
-        handle = entry_to_handle( entry );  /* make it a full handle */
-    }
-    LeaveCriticalSection( &gdi_section );
-
-    if (funcs)
-    {
-        if (!funcs->pGetObjectW)
-            SetLastError( ERROR_INVALID_HANDLE );
-        else if (buffer && ((ULONG_PTR)buffer >> 16) == 0) /* catch apps getting argument order wrong */
+        if (buffer && ((ULONG_PTR)buffer >> 16) == 0) /* catch apps getting argument order wrong */
             SetLastError( ERROR_NOACCESS );
         else
             result = funcs->pGetObjectW( handle, count, buffer );
@@ -1084,64 +1000,28 @@ INT WINAPI GetObjectW( HGDIOBJ handle, INT count, LPVOID buffer )
 }
 
 /***********************************************************************
- *           GetObjectType    (GDI32.@)
- */
-DWORD WINAPI GetObjectType( HGDIOBJ handle )
-{
-    struct gdi_handle_entry *entry;
-    DWORD result = 0;
-
-    EnterCriticalSection( &gdi_section );
-    if ((entry = handle_entry( handle ))) result = entry->type;
-    LeaveCriticalSection( &gdi_section );
-
-    TRACE("%p -> %u\n", handle, result );
-    if (!result) SetLastError( ERROR_INVALID_HANDLE );
-    return result;
-}
-
-/***********************************************************************
- *           GetCurrentObject    	(GDI32.@)
+ *           NtGdiGetDCObject    (win32u.@)
  *
  * Get the currently selected object of a given type in a device context.
- *
- * PARAMS
- *  hdc  [I] Device context to get the current object from
- *  type [I] Type of current object to get (OBJ_* defines from "wingdi.h")
- *
- * RETURNS
- *  Success: The current object of the given type selected in hdc.
- *  Failure: A NULL handle.
- *
- * NOTES
- * - only the following object types are supported:
- *| OBJ_PEN
- *| OBJ_BRUSH
- *| OBJ_PAL
- *| OBJ_FONT
- *| OBJ_BITMAP
  */
-HGDIOBJ WINAPI GetCurrentObject(HDC hdc,UINT type)
+HANDLE WINAPI NtGdiGetDCObject( HDC hdc, UINT type )
 {
     HGDIOBJ ret = 0;
-    DC * dc = get_dc_ptr( hdc );
+    DC *dc;
 
-    if (!dc) return 0;
+    if (!(dc = get_dc_ptr( hdc ))) return 0;
 
-    switch (type) {
-	case OBJ_EXTPEN: /* fall through */
-	case OBJ_PEN:	 ret = dc->hPen; break;
-	case OBJ_BRUSH:	 ret = dc->hBrush; break;
-	case OBJ_PAL:	 ret = dc->hPalette; break;
-	case OBJ_FONT:	 ret = dc->hFont; break;
-	case OBJ_BITMAP: ret = dc->hBitmap; break;
-
-	/* tests show that OBJ_REGION is explicitly ignored */
-	case OBJ_REGION: break;
-        default:
-            /* the SDK only mentions those above */
-            FIXME("(%p,%d): unknown type.\n",hdc,type);
-	    break;
+    switch (type)
+    {
+    case NTGDI_OBJ_EXTPEN: /* fall through */
+    case NTGDI_OBJ_PEN:    ret = dc->hPen; break;
+    case NTGDI_OBJ_BRUSH:  ret = dc->hBrush; break;
+    case NTGDI_OBJ_PAL:    ret = dc->hPalette; break;
+    case NTGDI_OBJ_FONT:   ret = dc->hFont; break;
+    case NTGDI_OBJ_SURF:   ret = dc->hBitmap; break;
+    default:
+        FIXME( "(%p, %d): unknown type.\n", hdc, type );
+        break;
     }
     release_dc_ptr( dc );
     return ret;
@@ -1149,53 +1029,17 @@ HGDIOBJ WINAPI GetCurrentObject(HDC hdc,UINT type)
 
 
 /***********************************************************************
- *           SelectObject    (GDI32.@)
- *
- * Select a Gdi object into a device context.
- *
- * PARAMS
- *  hdc  [I] Device context to associate the object with
- *  hObj [I] Gdi object to associate with hdc
- *
- * RETURNS
- *  Success: A non-NULL handle representing the previously selected object of
- *           the same type as hObj.
- *  Failure: A NULL object. If hdc is invalid, GetLastError() returns ERROR_INVALID_HANDLE.
- *           if hObj is not a valid object handle, no last error is set. In either
- *           case, hdc is unaffected by the call.
+ *           NtGdiUnrealizeObject    (win32u.@)
  */
-HGDIOBJ WINAPI SelectObject( HDC hdc, HGDIOBJ hObj )
-{
-    struct gdi_handle_entry *entry;
-    const struct gdi_obj_funcs *funcs = NULL;
-
-    TRACE( "(%p,%p)\n", hdc, hObj );
-
-    EnterCriticalSection( &gdi_section );
-    if ((entry = handle_entry( hObj )))
-    {
-        funcs = entry->funcs;
-        hObj = entry_to_handle( entry );  /* make it a full handle */
-    }
-    LeaveCriticalSection( &gdi_section );
-
-    if (funcs && funcs->pSelectObject) return funcs->pSelectObject( hObj, hdc );
-    return 0;
-}
-
-
-/***********************************************************************
- *           UnrealizeObject    (GDI32.@)
- */
-BOOL WINAPI UnrealizeObject( HGDIOBJ obj )
+BOOL WINAPI NtGdiUnrealizeObject( HGDIOBJ obj )
 {
     const struct gdi_obj_funcs *funcs = NULL;
-    struct gdi_handle_entry *entry;
+    GDI_HANDLE_ENTRY *entry;
 
     EnterCriticalSection( &gdi_section );
     if ((entry = handle_entry( obj )))
     {
-        funcs = entry->funcs;
+        funcs = entry_obj( entry )->funcs;
         obj = entry_to_handle( entry );  /* make it a full handle */
     }
     LeaveCriticalSection( &gdi_section );
@@ -1205,164 +1049,29 @@ BOOL WINAPI UnrealizeObject( HGDIOBJ obj )
 }
 
 
-/* Solid colors to enumerate */
-static const COLORREF solid_colors[] =
-{ RGB(0x00,0x00,0x00), RGB(0xff,0xff,0xff),
-RGB(0xff,0x00,0x00), RGB(0x00,0xff,0x00),
-RGB(0x00,0x00,0xff), RGB(0xff,0xff,0x00),
-RGB(0xff,0x00,0xff), RGB(0x00,0xff,0xff),
-RGB(0x80,0x00,0x00), RGB(0x00,0x80,0x00),
-RGB(0x80,0x80,0x00), RGB(0x00,0x00,0x80),
-RGB(0x80,0x00,0x80), RGB(0x00,0x80,0x80),
-RGB(0x80,0x80,0x80), RGB(0xc0,0xc0,0xc0)
-};
-
-
 /***********************************************************************
- *           EnumObjects    (GDI32.@)
+ *           NtGdiFlush    (win32u.@)
  */
-INT WINAPI EnumObjects( HDC hdc, INT nObjType,
-                            GOBJENUMPROC lpEnumFunc, LPARAM lParam )
-{
-    UINT i;
-    INT retval = 0;
-    LOGPEN pen;
-    LOGBRUSH brush;
-
-    TRACE("%p %d %p %08lx\n", hdc, nObjType, lpEnumFunc, lParam );
-    switch(nObjType)
-    {
-    case OBJ_PEN:
-        /* Enumerate solid pens */
-        for (i = 0; i < ARRAY_SIZE( solid_colors ); i++)
-        {
-            pen.lopnStyle   = PS_SOLID;
-            pen.lopnWidth.x = 1;
-            pen.lopnWidth.y = 0;
-            pen.lopnColor   = solid_colors[i];
-            retval = lpEnumFunc( &pen, lParam );
-            TRACE("solid pen %08x, ret=%d\n",
-                         solid_colors[i], retval);
-            if (!retval) break;
-        }
-        break;
-
-    case OBJ_BRUSH:
-        /* Enumerate solid brushes */
-        for (i = 0; i < ARRAY_SIZE( solid_colors ); i++)
-        {
-            brush.lbStyle = BS_SOLID;
-            brush.lbColor = solid_colors[i];
-            brush.lbHatch = 0;
-            retval = lpEnumFunc( &brush, lParam );
-            TRACE("solid brush %08x, ret=%d\n",
-                         solid_colors[i], retval);
-            if (!retval) break;
-        }
-
-        /* Now enumerate hatched brushes */
-        if (retval) for (i = HS_HORIZONTAL; i <= HS_DIAGCROSS; i++)
-        {
-            brush.lbStyle = BS_HATCHED;
-            brush.lbColor = RGB(0,0,0);
-            brush.lbHatch = i;
-            retval = lpEnumFunc( &brush, lParam );
-            TRACE("hatched brush %d, ret=%d\n",
-                         i, retval);
-            if (!retval) break;
-        }
-        break;
-
-    default:
-        /* FIXME: implement Win32 types */
-        WARN("(%d): Invalid type\n", nObjType );
-        break;
-    }
-    return retval;
-}
-
-
-/***********************************************************************
- *           SetObjectOwner    (GDI32.@)
- */
-void WINAPI SetObjectOwner( HGDIOBJ handle, HANDLE owner )
-{
-    /* Nothing to do */
-}
-
-/***********************************************************************
- *           GdiInitializeLanguagePack    (GDI32.@)
- */
-DWORD WINAPI GdiInitializeLanguagePack( DWORD arg )
-{
-    FIXME("stub\n");
-    return 0;
-}
-
-/***********************************************************************
- *           GdiFlush    (GDI32.@)
- */
-BOOL WINAPI GdiFlush(void)
+BOOL WINAPI NtGdiFlush(void)
 {
     return TRUE;  /* FIXME */
 }
 
 
-/***********************************************************************
- *           GdiGetBatchLimit    (GDI32.@)
- */
-DWORD WINAPI GdiGetBatchLimit(void)
-{
-    return 1;  /* FIXME */
-}
-
-
-/***********************************************************************
- *           GdiSetBatchLimit    (GDI32.@)
- */
-DWORD WINAPI GdiSetBatchLimit( DWORD limit )
-{
-    return 1; /* FIXME */
-}
-
-
 /*******************************************************************
- *      GetColorAdjustment [GDI32.@]
- *
- *
+ *           NtGdiGetColorAdjustment    (win32u.@)
  */
-BOOL WINAPI GetColorAdjustment(HDC hdc, LPCOLORADJUSTMENT lpca)
+BOOL WINAPI NtGdiGetColorAdjustment( HDC hdc, COLORADJUSTMENT *ca )
 {
-    FIXME("stub\n");
+    FIXME( "stub\n" );
     return FALSE;
 }
 
 /*******************************************************************
- *      GdiComment [GDI32.@]
- *
- *
+ *           NtGdiSetColorAdjustment    (win32u.@)
  */
-BOOL WINAPI GdiComment(HDC hdc, UINT cbSize, const BYTE *lpData)
+BOOL WINAPI NtGdiSetColorAdjustment( HDC hdc, const COLORADJUSTMENT *ca )
 {
-    DC *dc = get_dc_ptr(hdc);
-    BOOL ret = FALSE;
-
-    if(dc)
-    {
-        PHYSDEV physdev = GET_DC_PHYSDEV( dc, pGdiComment );
-        ret = physdev->funcs->pGdiComment( physdev, cbSize, lpData );
-        release_dc_ptr( dc );
-    }
-    return ret;
-}
-
-/*******************************************************************
- *      SetColorAdjustment [GDI32.@]
- *
- *
- */
-BOOL WINAPI SetColorAdjustment(HDC hdc, const COLORADJUSTMENT* lpca)
-{
-    FIXME("stub\n");
+    FIXME( "stub\n" );
     return FALSE;
 }
