@@ -18,7 +18,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "gdi_private.h"
+#include "ntgdi_private.h"
 
 /***********************************************************************
  *      __wine_get_vulkan_driver  (GDI32.@)
