@@ -227,6 +227,7 @@ static HRESULT WINAPI Client_get_accRole(IAccessible *iface, VARIANT varID, VARI
 static HRESULT WINAPI Client_get_accState(IAccessible *iface, VARIANT varID, VARIANT *pvarState)
 {
     Client *This = impl_from_Client(iface);
+    GUITHREADINFO info;
     LONG style;
 
     TRACE("(%p)->(%s %p)\n", This, debugstr_variant(&varID), pvarState);
@@ -244,7 +245,9 @@ static HRESULT WINAPI Client_get_accState(IAccessible *iface, VARIANT varID, VAR
         V_I4(pvarState) |= STATE_SYSTEM_UNAVAILABLE;
     else if(IsWindow(This->hwnd))
         V_I4(pvarState) |= STATE_SYSTEM_FOCUSABLE;
-    if(GetFocus() == This->hwnd)
+
+    info.cbSize = sizeof(info);
+    if(GetGUIThreadInfo(0, &info) && info.hwndFocus == This->hwnd)
         V_I4(pvarState) |= STATE_SYSTEM_FOCUSED;
     if(!(style & WS_VISIBLE))
         V_I4(pvarState) |= STATE_SYSTEM_INVISIBLE;
@@ -300,11 +303,37 @@ static HRESULT WINAPI Client_get_accKeyboardShortcut(IAccessible *iface,
     return S_OK;
 }
 
-static HRESULT WINAPI Client_get_accFocus(IAccessible *iface, VARIANT *pvarID)
+static HRESULT WINAPI Client_get_accFocus(IAccessible *iface, VARIANT *focus)
 {
     Client *This = impl_from_Client(iface);
-    FIXME("(%p)->(%p)\n", This, pvarID);
-    return E_NOTIMPL;
+    GUITHREADINFO info;
+
+    TRACE("(%p)->(%p)\n", This, focus);
+
+    V_VT(focus) = VT_EMPTY;
+    info.cbSize = sizeof(info);
+    if(GetGUIThreadInfo(0, &info) && info.hwndFocus) {
+        if(info.hwndFocus == This->hwnd) {
+            V_VT(focus) = VT_I4;
+            V_I4(focus) = CHILDID_SELF;
+        }
+        else if(IsChild(This->hwnd, info.hwndFocus)) {
+            IDispatch *disp;
+            HRESULT hr;
+
+            hr = AccessibleObjectFromWindow(info.hwndFocus, OBJID_WINDOW,
+                    &IID_IDispatch, (void**)&disp);
+            if(FAILED(hr))
+                return hr;
+            if(!disp)
+                return E_FAIL;
+
+            V_VT(focus) = VT_DISPATCH;
+            V_DISPATCH(focus) = disp;
+        }
+    }
+
+    return S_OK;
 }
 
 static HRESULT WINAPI Client_get_accSelection(IAccessible *iface, VARIANT *pvarID)
