@@ -170,6 +170,11 @@ static ULONG WINAPI source_async_command_Release(IUnknown *iface)
     {
         if (command->op == SOURCE_ASYNC_START)
             PropVariantClear(&command->u.start.position);
+        else if (command->op == SOURCE_ASYNC_REQUEST_SAMPLE)
+        {
+            if (command->u.request_sample.token)
+                IUnknown_Release(command->u.request_sample.token);
+        }
         free(command);
     }
 
@@ -467,8 +472,6 @@ static void wait_on_sample(struct media_stream *stream, IUnknown *token)
 
             case WG_PARSER_EVENT_EOS:
                 stream->eos = TRUE;
-                if (token)
-                    IUnknown_Release(token);
                 IMFMediaEventQueue_QueueEventParamVar(stream->event_queue, MEEndOfStream, &GUID_NULL, S_OK, &empty_var);
                 dispatch_end_of_presentation(stream->parent_source);
                 return;
@@ -527,6 +530,11 @@ static DWORD CALLBACK read_thread(void *arg)
 {
     struct media_source *source = arg;
     IMFByteStream *byte_stream = source->byte_stream;
+    uint32_t buffer_size = 0;
+    uint64_t file_size;
+    void *data = NULL;
+
+    IMFByteStream_GetLength(byte_stream, &file_size);
 
     TRACE("Starting read thread for media source %p.\n", source);
 
@@ -536,18 +544,33 @@ static DWORD CALLBACK read_thread(void *arg)
         ULONG ret_size;
         uint32_t size;
         HRESULT hr;
-        void *data;
 
-        if (!unix_funcs->wg_parser_get_read_request(source->wg_parser, &data, &offset, &size))
+        if (!unix_funcs->wg_parser_get_next_read_offset(source->wg_parser, &offset, &size))
             continue;
 
+        if (offset >= file_size)
+            size = 0;
+        else if (offset + size >= file_size)
+            size = file_size - offset;
+
+        if (size > buffer_size)
+        {
+            buffer_size = size;
+            data = realloc(data, size);
+        }
+
+        ret_size = 0;
+
         if (SUCCEEDED(hr = IMFByteStream_SetCurrentPosition(byte_stream, offset)))
             hr = IMFByteStream_Read(byte_stream, data, size, &ret_size);
-        if (SUCCEEDED(hr) && ret_size != size)
+        if (FAILED(hr))
+            ERR("Failed to read %u bytes at offset %I64u, hr %#x.\n", size, offset, hr);
+        else if (ret_size != size)
             ERR("Unexpected short read: requested %u bytes, got %u.\n", size, ret_size);
-        unix_funcs->wg_parser_complete_read_request(source->wg_parser, SUCCEEDED(hr));
+        unix_funcs->wg_parser_push_data(source->wg_parser, SUCCEEDED(hr) ? data : NULL, ret_size);
     }
 
+    free(data);
     TRACE("Media source is shutting down; exiting.\n");
     return 0;
 }
@@ -556,7 +579,7 @@ static HRESULT WINAPI media_stream_QueryInterface(IMFMediaStream *iface, REFIID
 {
     struct media_stream *stream = impl_from_IMFMediaStream(iface);
 
-    TRACE("(%p)->(%s %p)\n", stream, debugstr_guid(riid), out);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(riid), out);
 
     if (IsEqualIID(riid, &IID_IMFMediaStream) ||
         IsEqualIID(riid, &IID_IMFMediaEventGenerator) ||
@@ -580,7 +603,7 @@ static ULONG WINAPI media_stream_AddRef(IMFMediaStream *iface)
     struct media_stream *stream = impl_from_IMFMediaStream(iface);
     ULONG ref = InterlockedIncrement(&stream->ref);
 
-    TRACE("(%p) ref=%u\n", stream, ref);
+    TRACE("%p, refcount %u.\n", iface, ref);
 
     return ref;
 }
@@ -588,10 +611,9 @@ static ULONG WINAPI media_stream_AddRef(IMFMediaStream *iface)
 static ULONG WINAPI media_stream_Release(IMFMediaStream *iface)
 {
     struct media_stream *stream = impl_from_IMFMediaStream(iface);
-
     ULONG ref = InterlockedDecrement(&stream->ref);
 
-    TRACE("(%p) ref=%u\n", stream, ref);
+    TRACE("%p, refcount %u.\n", iface, ref);
 
     if (!ref)
     {
@@ -607,7 +629,7 @@ static HRESULT WINAPI media_stream_GetEvent(IMFMediaStream *iface, DWORD flags,
 {
     struct media_stream *stream = impl_from_IMFMediaStream(iface);
 
-    TRACE("(%p)->(%#x, %p)\n", stream, flags, event);
+    TRACE("%p, %#x, %p.\n", iface, flags, event);
 
     return IMFMediaEventQueue_GetEvent(stream->event_queue, flags, event);
 }
@@ -616,7 +638,7 @@ static HRESULT WINAPI media_stream_BeginGetEvent(IMFMediaStream *iface, IMFAsync
 {
     struct media_stream *stream = impl_from_IMFMediaStream(iface);
 
-    TRACE("(%p)->(%p, %p)\n", stream, callback, state);
+    TRACE("%p, %p, %p.\n", iface, callback, state);
 
     return IMFMediaEventQueue_BeginGetEvent(stream->event_queue, callback, state);
 }
@@ -625,7 +647,7 @@ static HRESULT WINAPI media_stream_EndGetEvent(IMFMediaStream *iface, IMFAsyncRe
 {
     struct media_stream *stream = impl_from_IMFMediaStream(iface);
 
-    TRACE("(%p)->(%p, %p)\n", stream, result, event);
+    TRACE("%p, %p, %p.\n", stream, result, event);
 
     return IMFMediaEventQueue_EndGetEvent(stream->event_queue, result, event);
 }
@@ -635,7 +657,7 @@ static HRESULT WINAPI media_stream_QueueEvent(IMFMediaStream *iface, MediaEventT
 {
     struct media_stream *stream = impl_from_IMFMediaStream(iface);
 
-    TRACE("(%p)->(%d, %s, %#x, %p)\n", stream, event_type, debugstr_guid(ext_type), hr, value);
+    TRACE("%p, %d, %s, %#x, %p.\n", iface, event_type, debugstr_guid(ext_type), hr, value);
 
     return IMFMediaEventQueue_QueueEventParamVar(stream->event_queue, event_type, ext_type, hr, value);
 }
@@ -644,7 +666,7 @@ static HRESULT WINAPI media_stream_GetMediaSource(IMFMediaStream *iface, IMFMedi
 {
     struct media_stream *stream = impl_from_IMFMediaStream(iface);
 
-    TRACE("(%p)->(%p)\n", stream, source);
+    TRACE("%p, %p.\n", iface, source);
 
     if (stream->state == STREAM_SHUTDOWN)
         return MF_E_SHUTDOWN;
@@ -659,7 +681,7 @@ static HRESULT WINAPI media_stream_GetStreamDescriptor(IMFMediaStream* iface, IM
 {
     struct media_stream *stream = impl_from_IMFMediaStream(iface);
 
-    TRACE("(%p)->(%p)\n", stream, descriptor);
+    TRACE("%p, %p.\n", iface, descriptor);
 
     if (stream->state == STREAM_SHUTDOWN)
         return MF_E_SHUTDOWN;
@@ -676,7 +698,7 @@ static HRESULT WINAPI media_stream_RequestSample(IMFMediaStream *iface, IUnknown
     struct source_async_command *command;
     HRESULT hr;
 
-    TRACE("(%p)->(%p)\n", iface, token);
+    TRACE("%p, %p.\n", iface, token);
 
     if (stream->state == STREAM_SHUTDOWN)
         return MF_E_SHUTDOWN;
@@ -731,6 +753,12 @@ static HRESULT new_media_stream(struct media_source *source,
     object->IMFMediaStream_iface.lpVtbl = &media_stream_vtbl;
     object->ref = 1;
 
+    if (FAILED(hr = MFCreateEventQueue(&object->event_queue)))
+    {
+        free(object);
+        return hr;
+    }
+
     IMFMediaSource_AddRef(&source->IMFMediaSource_iface);
     object->parent_source = source;
     object->stream_id = stream_id;
@@ -739,26 +767,17 @@ static HRESULT new_media_stream(struct media_source *source,
     object->eos = FALSE;
     object->wg_stream = wg_stream;
 
-    if (FAILED(hr = MFCreateEventQueue(&object->event_queue)))
-        goto fail;
+    TRACE("Created stream object %p.\n", object);
 
-    TRACE("->(%p)\n", object);
     *out_stream = object;
 
     return S_OK;
-
-fail:
-    WARN("Failed to construct media stream, hr %#x.\n", hr);
-
-    IMFMediaStream_Release(&object->IMFMediaStream_iface);
-    return hr;
 }
 
 static HRESULT media_stream_init_desc(struct media_stream *stream)
 {
     IMFMediaTypeHandler *type_handler = NULL;
-    IMFMediaType **stream_types = NULL;
-    IMFMediaType *stream_type = NULL;
+    IMFMediaType *stream_types[6];
     struct wg_format format;
     DWORD type_count = 0;
     unsigned int i;
@@ -784,8 +803,6 @@ static HRESULT media_stream_init_desc(struct media_stream *stream)
 
         IMFMediaType_GetGUID(base_type, &MF_MT_SUBTYPE, &base_subtype);
 
-        stream_types = malloc(sizeof(IMFMediaType *) * (ARRAY_SIZE(video_types) + 1));
-
         stream_types[0] = base_type;
         type_count = 1;
 
@@ -806,15 +823,36 @@ static HRESULT media_stream_init_desc(struct media_stream *stream)
                 goto done;
         }
     }
-    else
+    else if (format.major_type == WG_MAJOR_TYPE_AUDIO)
     {
-        stream_type = mf_media_type_from_wg_format(&format);
-        if (stream_type)
+        /* Expose at least one PCM and one floating point type for the
+           consumer to pick from. */
+        static const enum wg_audio_format audio_types[] =
         {
-            stream_types = &stream_type;
-            type_count = 1;
+            WG_AUDIO_FORMAT_S16LE,
+            WG_AUDIO_FORMAT_F32LE,
+        };
+
+        stream_types[0] = mf_media_type_from_wg_format(&format);
+        type_count = 1;
+
+        for (i = 0; i < ARRAY_SIZE(audio_types); i++)
+        {
+            struct wg_format new_format;
+            if (format.u.audio.format == audio_types[i])
+                continue;
+            new_format = format;
+            new_format.u.audio.format = audio_types[i];
+            stream_types[type_count++] = mf_media_type_from_wg_format(&new_format);
         }
     }
+    else
+    {
+        if ((stream_types[0] = mf_media_type_from_wg_format(&format)))
+            type_count = 1;
+    }
+
+    assert(type_count <= ARRAY_SIZE(stream_types));
 
     if (!type_count)
     {
@@ -826,18 +864,22 @@ static HRESULT media_stream_init_desc(struct media_stream *stream)
         goto done;
 
     if (FAILED(hr = IMFStreamDescriptor_GetMediaTypeHandler(stream->descriptor, &type_handler)))
+    {
+        IMFStreamDescriptor_Release(stream->descriptor);
         goto done;
+    }
 
     if (FAILED(hr = IMFMediaTypeHandler_SetCurrentMediaType(type_handler, stream_types[0])))
+    {
+        IMFStreamDescriptor_Release(stream->descriptor);
         goto done;
+    }
 
 done:
     if (type_handler)
         IMFMediaTypeHandler_Release(type_handler);
     for (i = 0; i < type_count; i++)
         IMFMediaType_Release(stream_types[i]);
-    if (stream_types != &stream_type)
-        free(stream_types);
     return hr;
 }
 
@@ -917,13 +959,7 @@ static HRESULT WINAPI media_source_rate_support_GetSlowestRate(IMFRateSupport *i
 {
     TRACE("%p, %d, %d, %p.\n", iface, direction, thin, rate);
 
-    if (direction == MFRATE_REVERSE)
-        return MF_E_REVERSE_UNSUPPORTED;
-
-    if (thin)
-        return MF_E_THINNING_UNSUPPORTED;
-
-    *rate = 1.0f;
+    *rate = 0.0f;
 
     return S_OK;
 }
@@ -932,31 +968,20 @@ static HRESULT WINAPI media_source_rate_support_GetFastestRate(IMFRateSupport *i
 {
     TRACE("%p, %d, %d, %p.\n", iface, direction, thin, rate);
 
-    if (direction == MFRATE_REVERSE)
-        return MF_E_REVERSE_UNSUPPORTED;
-
-    if (thin)
-        return MF_E_THINNING_UNSUPPORTED;
-
-    *rate = 1.0f;
+    *rate = direction == MFRATE_FORWARD ? 1e6f : -1e6f;
 
     return S_OK;
 }
 
-static HRESULT WINAPI media_source_rate_support_IsRateSupported(IMFRateSupport *iface, BOOL thin, float rate, float *nearest_support_rate)
+static HRESULT WINAPI media_source_rate_support_IsRateSupported(IMFRateSupport *iface, BOOL thin, float rate,
+        float *nearest_rate)
 {
-    TRACE("%p, %d, %f, %p.\n", iface, thin, rate, nearest_support_rate);
+    TRACE("%p, %d, %f, %p.\n", iface, thin, rate, nearest_rate);
 
-    if (rate < 0.0f)
-        return MF_E_REVERSE_UNSUPPORTED;
+    if (nearest_rate)
+        *nearest_rate = rate;
 
-    if (thin)
-        return MF_E_THINNING_UNSUPPORTED;
-
-    if (nearest_support_rate)
-        *nearest_support_rate = 1.0f;
-
-    return rate == 1.0f ? S_OK : MF_E_UNSUPPORTED_RATE;
+    return rate >= -1e6f && rate <= 1e6f ? S_OK : MF_E_UNSUPPORTED_RATE;
 }
 
 static const IMFRateSupportVtbl media_source_rate_support_vtbl =
@@ -1028,7 +1053,7 @@ static HRESULT WINAPI media_source_QueryInterface(IMFMediaSource *iface, REFIID
 {
     struct media_source *source = impl_from_IMFMediaSource(iface);
 
-    TRACE("(%p)->(%s %p)\n", source, debugstr_guid(riid), out);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(riid), out);
 
     if (IsEqualIID(riid, &IID_IMFMediaSource) ||
         IsEqualIID(riid, &IID_IMFMediaEventGenerator) ||
@@ -1042,7 +1067,7 @@ static HRESULT WINAPI media_source_QueryInterface(IMFMediaSource *iface, REFIID
     }
     else
     {
-        FIXME("(%s, %p)\n", debugstr_guid(riid), out);
+        FIXME("%s, %p.\n", debugstr_guid(riid), out);
         *out = NULL;
         return E_NOINTERFACE;
     }
@@ -1056,7 +1081,7 @@ static ULONG WINAPI media_source_AddRef(IMFMediaSource *iface)
     struct media_source *source = impl_from_IMFMediaSource(iface);
     ULONG ref = InterlockedIncrement(&source->ref);
 
-    TRACE("(%p) ref=%u\n", source, ref);
+    TRACE("%p, refcount %u.\n", iface, ref);
 
     return ref;
 }
@@ -1066,7 +1091,7 @@ static ULONG WINAPI media_source_Release(IMFMediaSource *iface)
     struct media_source *source = impl_from_IMFMediaSource(iface);
     ULONG ref = InterlockedDecrement(&source->ref);
 
-    TRACE("(%p) ref=%u\n", source, ref);
+    TRACE("%p, refcount %u.\n", iface, ref);
 
     if (!ref)
     {
@@ -1082,7 +1107,7 @@ static HRESULT WINAPI media_source_GetEvent(IMFMediaSource *iface, DWORD flags,
 {
     struct media_source *source = impl_from_IMFMediaSource(iface);
 
-    TRACE("(%p)->(%#x, %p)\n", source, flags, event);
+    TRACE("%p, %#x, %p.\n", iface, flags, event);
 
     return IMFMediaEventQueue_GetEvent(source->event_queue, flags, event);
 }
@@ -1091,7 +1116,7 @@ static HRESULT WINAPI media_source_BeginGetEvent(IMFMediaSource *iface, IMFAsync
 {
     struct media_source *source = impl_from_IMFMediaSource(iface);
 
-    TRACE("(%p)->(%p, %p)\n", source, callback, state);
+    TRACE("%p, %p, %p.\n", iface, callback, state);
 
     return IMFMediaEventQueue_BeginGetEvent(source->event_queue, callback, state);
 }
@@ -1100,7 +1125,7 @@ static HRESULT WINAPI media_source_EndGetEvent(IMFMediaSource *iface, IMFAsyncRe
 {
     struct media_source *source = impl_from_IMFMediaSource(iface);
 
-    TRACE("(%p)->(%p, %p)\n", source, result, event);
+    TRACE("%p, %p, %p.\n", iface, result, event);
 
     return IMFMediaEventQueue_EndGetEvent(source->event_queue, result, event);
 }
@@ -1110,7 +1135,7 @@ static HRESULT WINAPI media_source_QueueEvent(IMFMediaSource *iface, MediaEventT
 {
     struct media_source *source = impl_from_IMFMediaSource(iface);
 
-    TRACE("(%p)->(%d, %s, %#x, %p)\n", source, event_type, debugstr_guid(ext_type), hr, value);
+    TRACE("%p, %d, %s, %#x, %p.\n", iface, event_type, debugstr_guid(ext_type), hr, value);
 
     return IMFMediaEventQueue_QueueEventParamVar(source->event_queue, event_type, ext_type, hr, value);
 }
@@ -1119,7 +1144,7 @@ static HRESULT WINAPI media_source_GetCharacteristics(IMFMediaSource *iface, DWO
 {
     struct media_source *source = impl_from_IMFMediaSource(iface);
 
-    TRACE("(%p)->(%p)\n", source, characteristics);
+    TRACE("%p, %p.\n", iface, characteristics);
 
     if (source->state == SOURCE_SHUTDOWN)
         return MF_E_SHUTDOWN;
@@ -1133,7 +1158,7 @@ static HRESULT WINAPI media_source_CreatePresentationDescriptor(IMFMediaSource *
 {
     struct media_source *source = impl_from_IMFMediaSource(iface);
 
-    TRACE("(%p)->(%p)\n", source, descriptor);
+    TRACE("%p, %p.\n", iface, descriptor);
 
     if (source->state == SOURCE_SHUTDOWN)
         return MF_E_SHUTDOWN;
@@ -1148,7 +1173,7 @@ static HRESULT WINAPI media_source_Start(IMFMediaSource *iface, IMFPresentationD
     struct source_async_command *command;
     HRESULT hr;
 
-    TRACE("(%p)->(%p, %p, %p)\n", source, descriptor, time_format, position);
+    TRACE("%p, %p, %p, %p.\n", iface, descriptor, time_format, position);
 
     if (source->state == SOURCE_SHUTDOWN)
         return MF_E_SHUTDOWN;
@@ -1174,7 +1199,7 @@ static HRESULT WINAPI media_source_Stop(IMFMediaSource *iface)
     struct source_async_command *command;
     HRESULT hr;
 
-    TRACE("(%p)\n", source);
+    TRACE("%p.\n", iface);
 
     if (source->state == SOURCE_SHUTDOWN)
         return MF_E_SHUTDOWN;
@@ -1189,7 +1214,7 @@ static HRESULT WINAPI media_source_Pause(IMFMediaSource *iface)
 {
     struct media_source *source = impl_from_IMFMediaSource(iface);
 
-    FIXME("(%p): stub\n", source);
+    FIXME("%p: stub\n", iface);
 
     if (source->state == SOURCE_SHUTDOWN)
         return MF_E_SHUTDOWN;
@@ -1202,7 +1227,7 @@ static HRESULT WINAPI media_source_Shutdown(IMFMediaSource *iface)
     struct media_source *source = impl_from_IMFMediaSource(iface);
     unsigned int i;
 
-    TRACE("(%p)\n", source);
+    TRACE("%p.\n", iface);
 
     if (source->state == SOURCE_SHUTDOWN)
         return MF_E_SHUTDOWN;
@@ -1211,12 +1236,13 @@ static HRESULT WINAPI media_source_Shutdown(IMFMediaSource *iface)
 
     unix_funcs->wg_parser_disconnect(source->wg_parser);
 
-    if (source->pres_desc)
-        IMFPresentationDescriptor_Release(source->pres_desc);
-    if (source->event_queue)
-        IMFMediaEventQueue_Shutdown(source->event_queue);
-    if (source->byte_stream)
-        IMFByteStream_Release(source->byte_stream);
+    source->read_thread_shutdown = true;
+    WaitForSingleObject(source->read_thread, INFINITE);
+    CloseHandle(source->read_thread);
+
+    IMFPresentationDescriptor_Release(source->pres_desc);
+    IMFMediaEventQueue_Shutdown(source->event_queue);
+    IMFByteStream_Release(source->byte_stream);
 
     for (i = 0; i < source->stream_count; i++)
     {
@@ -1224,30 +1250,18 @@ static HRESULT WINAPI media_source_Shutdown(IMFMediaSource *iface)
 
         stream->state = STREAM_SHUTDOWN;
 
-        if (stream->event_queue)
-            IMFMediaEventQueue_Shutdown(stream->event_queue);
-        if (stream->descriptor)
-            IMFStreamDescriptor_Release(stream->descriptor);
-        if (stream->parent_source)
-            IMFMediaSource_Release(&stream->parent_source->IMFMediaSource_iface);
+        IMFMediaEventQueue_Shutdown(stream->event_queue);
+        IMFStreamDescriptor_Release(stream->descriptor);
+        IMFMediaSource_Release(&stream->parent_source->IMFMediaSource_iface);
 
         IMFMediaStream_Release(&stream->IMFMediaStream_iface);
     }
 
-    if (source->read_thread)
-    {
-        source->read_thread_shutdown = true;
-        WaitForSingleObject(source->read_thread, INFINITE);
-        CloseHandle(source->read_thread);
-    }
-
     unix_funcs->wg_parser_destroy(source->wg_parser);
 
-    if (source->stream_count)
-        free(source->streams);
+    free(source->streams);
 
-    if (source->async_commands_queue)
-        MFUnlockWorkQueue(source->async_commands_queue);
+    MFUnlockWorkQueue(source->async_commands_queue);
 
     return S_OK;
 }
@@ -1272,6 +1286,7 @@ static const IMFMediaSourceVtbl IMFMediaSource_vtbl =
 static HRESULT media_source_constructor(IMFByteStream *bytestream, struct media_source **out_media_source)
 {
     IMFStreamDescriptor **descriptors = NULL;
+    unsigned int stream_count = UINT_MAX;
     struct media_source *object;
     UINT64 total_pres_time = 0;
     struct wg_parser *parser;
@@ -1313,8 +1328,6 @@ static HRESULT media_source_constructor(IMFByteStream *bytestream, struct media_
     if (FAILED(hr = MFAllocateWorkQueue(&object->async_commands_queue)))
         goto fail;
 
-    object->read_thread = CreateThread(NULL, 0, read_thread, object, 0, NULL);
-
     if (!(parser = unix_funcs->wg_decodebin_parser_create()))
     {
         hr = E_OUTOFMEMORY;
@@ -1322,6 +1335,8 @@ static HRESULT media_source_constructor(IMFByteStream *bytestream, struct media_
     }
     object->wg_parser = parser;
 
+    object->read_thread = CreateThread(NULL, 0, read_thread, object, 0, NULL);
+
     object->state = SOURCE_OPENING;
 
     if (FAILED(hr = unix_funcs->wg_parser_connect(parser, file_size)))
@@ -1335,15 +1350,15 @@ static HRESULT media_source_constructor(IMFByteStream *bytestream, struct media_
      * leak occurs with native. */
     unix_funcs->wg_parser_set_unlimited_buffering(parser);
 
-    object->stream_count = unix_funcs->wg_parser_get_stream_count(parser);
+    stream_count = unix_funcs->wg_parser_get_stream_count(parser);
 
-    if (!(object->streams = calloc(object->stream_count, sizeof(*object->streams))))
+    if (!(object->streams = calloc(stream_count, sizeof(*object->streams))))
     {
         hr = E_OUTOFMEMORY;
         goto fail;
     }
 
-    for (i = 0; i < object->stream_count; ++i)
+    for (i = 0; i < stream_count; ++i)
     {
         if (FAILED(hr = new_media_stream(object, unix_funcs->wg_parser_get_stream(parser, i), i, &object->streams[i])))
             goto fail;
@@ -1351,9 +1366,13 @@ static HRESULT media_source_constructor(IMFByteStream *bytestream, struct media_
         if (FAILED(hr = media_stream_init_desc(object->streams[i])))
         {
             ERR("Failed to finish initialization of media stream %p, hr %x.\n", object->streams[i], hr);
-            IMFMediaStream_Release(&object->streams[i]->IMFMediaStream_iface);
+            IMFMediaSource_Release(&object->streams[i]->parent_source->IMFMediaSource_iface);
+            IMFMediaEventQueue_Release(object->streams[i]->event_queue);
+            free(object->streams[i]);
             goto fail;
         }
+
+        object->stream_count++;
     }
 
     /* init presentation descriptor */
@@ -1390,8 +1409,39 @@ static HRESULT media_source_constructor(IMFByteStream *bytestream, struct media_
     fail:
     WARN("Failed to construct MFMediaSource, hr %#x.\n", hr);
 
-    free(descriptors);
-    IMFMediaSource_Release(&object->IMFMediaSource_iface);
+    if (descriptors)
+    {
+        for (i = 0; i < object->stream_count; i++)
+            IMFStreamDescriptor_Release(descriptors[i]);
+        free(descriptors);
+    }
+    for (i = 0; i < object->stream_count; i++)
+    {
+        struct media_stream *stream = object->streams[i];
+
+        IMFMediaEventQueue_Release(stream->event_queue);
+        IMFStreamDescriptor_Release(stream->descriptor);
+        IMFMediaSource_Release(&stream->parent_source->IMFMediaSource_iface);
+
+        free(stream);
+    }
+    free(object->streams);
+    if (stream_count != UINT_MAX)
+        unix_funcs->wg_parser_disconnect(object->wg_parser);
+    if (object->read_thread)
+    {
+        object->read_thread_shutdown = true;
+        WaitForSingleObject(object->read_thread, INFINITE);
+        CloseHandle(object->read_thread);
+    }
+    if (object->wg_parser)
+        unix_funcs->wg_parser_destroy(object->wg_parser);
+    if (object->async_commands_queue)
+        MFUnlockWorkQueue(object->async_commands_queue);
+    if (object->event_queue)
+        IMFMediaEventQueue_Release(object->event_queue);
+    IMFByteStream_Release(object->byte_stream);
+    free(object);
     return hr;
 }
 
@@ -1728,7 +1778,7 @@ static HRESULT WINAPI winegstreamer_stream_handler_callback_GetParameters(IMFAsy
 static HRESULT winegstreamer_stream_handler_create_object(struct winegstreamer_stream_handler *This, WCHAR *url, IMFByteStream *stream, DWORD flags,
                                             IPropertyStore *props, IUnknown **out_object, MF_OBJECT_TYPE *out_obj_type)
 {
-    TRACE("(%p %s %p %u %p %p %p)\n", This, debugstr_w(url), stream, flags, props, out_object, out_obj_type);
+    TRACE("%p, %s, %p, %u, %p, %p, %p.\n", This, debugstr_w(url), stream, flags, props, out_object, out_obj_type);
 
     if (flags & MF_RESOLUTION_MEDIASOURCE)
     {
