@@ -462,7 +462,7 @@ void init_directories( struct fd *intl_fd )
 
     /* sessions */
     create_session( 0 );
-    create_session( 1 );
+    create_session( default_session_id );
 
     /* object types */
 
