@@ -192,11 +192,11 @@ static void write_namespace_end(FILE *header, struct namespace *namespace)
 {
     if(is_global_namespace(namespace)) {
         if(use_abi_namespace)
-            write_line(header, -1, "}", namespace->name);
+            write_line(header, -1, "}");
         return;
     }
 
-    write_line(header, -1, "}", namespace->name);
+    write_line(header, -1, "}");
     write_namespace_end(header, namespace->parent);
 }
 
@@ -488,7 +488,7 @@ void write_type_left(FILE *h, const decl_spec_t *ds, enum name_type name_type, i
         fprintf(h, "%s", type_get_qualified_name(t, name_type));
         break;
       case TYPE_RUNTIMECLASS:
-        fprintf(h, "%s", type_get_name(type_runtimeclass_get_default_iface(t), name_type));
+        fprintf(h, "%s", type_get_name(type_runtimeclass_get_default_iface(t, TRUE), name_type));
         break;
       case TYPE_DELEGATE:
         fprintf(h, "%s", type_get_qualified_name(type_delegate_get_iface(t), name_type));
@@ -872,11 +872,25 @@ static void write_generic_handle_routines(FILE *header)
 
 static void write_typedef(FILE *header, type_t *type, int declonly)
 {
-  type_t *t = type_alias_get_aliasee_type(type);
-  if (winrt_mode && t->namespace && !is_global_namespace(t->namespace)) return;
-  fprintf(header, "typedef ");
-  write_type_v(header, type_alias_get_aliasee(type), FALSE, declonly, type->name, NAME_DEFAULT);
-  fprintf(header, ";\n");
+    type_t *t = type_alias_get_aliasee_type(type);
+    if (winrt_mode && t->namespace && !is_global_namespace(t->namespace))
+    {
+        fprintf(header, "#ifdef __cplusplus\n");
+        write_namespace_start(header, t->namespace);
+        indent(header, 0);
+    }
+    fprintf(header, "typedef ");
+    write_type_v(header, type_alias_get_aliasee(type), FALSE, declonly, type->name, NAME_DEFAULT);
+    fprintf(header, ";\n");
+    if (winrt_mode && t->namespace && !is_global_namespace(t->namespace))
+    {
+        write_namespace_end(header, t->namespace);
+        fprintf(header, "#else /* __cplusplus */\n");
+        fprintf(header, "typedef ");
+        write_type_v(header, type_alias_get_aliasee(type), FALSE, declonly, type->c_name, NAME_C);
+        fprintf(header, ";\n");
+        fprintf(header, "#endif /* __cplusplus */\n\n");
+    }
 }
 
 int is_const_decl(const var_t *var)
@@ -1534,7 +1548,7 @@ static void write_parameterized_type_forward(FILE *header, type_t *type)
 static void write_parameterized_implementation(FILE *header, type_t *type, int declonly)
 {
     const statement_t *stmt;
-    typeref_list_t *params = params = type->details.parameterized.params;
+    typeref_list_t *params = type->details.parameterized.params;
     typeref_t *ref;
     type_t *iface = type->details.parameterized.type, *base;
     char *args = NULL;
